#!/bin/bash
#
# orakafka_stream.sh
#
# Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      orakafka_stream.sh 
#
#    DESCRIPTION
#    This is the external table preprocessor script on the database for 
#    Oracle SQL Access to Kafka.

#    The orakafka.sh CLI installs this script in
#    <path>/app_data/clusters/<EXECDIR>/orakafka_stream.sh 
#    
#    where
#       <path> is the absolute path of ora_kafka_home/ directory (default)
#                 example: /u01/ora_kafka_home 
#              or is path to app_data/ directory (non-default/RAC)
#
#    RUNTIME REQUIREMENTS
#    This script requires the use of Oracle Java 8 JRE.  See JAVA_HOME
#    requirements below. 
#   
#
#    Usage: orakafka_stream <locationFile>
#   
#
#    Location files can be found in the default directory of the external
#    table created by ORA_KAFKA PL/SQL package.
# 

export PATH=/usr/bin:/bin

#============== Set ENABLE_SSL_DEBUG (OPTIONAL) =================
# Uncomment the following line to enable SSL/TLS debugging
#================================================================
#ENABLE_SSL_DEBUG=true

###############################################################################
#
# discover and set EXECUTE_DIR to the absolute path of the kit installation 
# directory
#
# get the name of this script from BASH_SOURCE 
# or from $0 (if BASH_SOURCE is null or empty)
this="${BASH_SOURCE:-$0}"
#
# get the current directory without symbolic links
#
# cd -P: The -P option means to use the physical directory structure 
#        instead of following symbolic links
#    '--' indicates end of options to the builtin command
#         See Bash Ref Manual: Shell Builtin commands for more info on '--'
# pwd -P: The -P ensures that the printed pathname will not contain any 
#         symbolic links
#
EXECUTE_DIR=$(cd -P -- "$(dirname -- "$this")" && pwd -P)
LOC_FILE="$1"
LOG_FILE="${LOC_FILE}".pp.out

#==============================================================================
#=============== SET PRODUCT_KIT_HOME (Required) 
#==============================================================================
# get PRODUCT_KIT_HOME 
# this script is generated by orakafka_distro_install.sh during installation
source ${EXECUTE_DIR}/../../scripts/configure_kit_home.sh

if [[ -d "${PRODUCT_KIT_HOME}/jlib/" ]]; then
  echo "SUCCESS: PRODUCT_KIT_HOME is valid." > "${LOG_FILE}"
else
  echo "FAILURE: PRODUCT_KIT_HOME=${PRODUCT_KIT_HOME} path is invalid." > "${LOG_FILE}"
  exit 1
fi

#==============================================================================
#=============== SET JAVA_HOME (OPTIONAL)====================================== 
#==============================================================================
# To set JAVA_HOME for the stream, run the following script:
# <ora-kafka-home>/orakafka/bin/orakafka.sh set_java_home -p <java-home-path>
# where <java-home-path> should be a valid JDK/JRE installation directory
# and the java version should be 1.8 or higher.

source ${EXECUTE_DIR}/../../scripts/configure_java.sh

if [[ -x "$JAVA_HOME/bin/java" ]]; then
  echo "SUCCESS: Java executable found in JAVA_HOME, JAVA_HOME path is valid." > "${LOG_FILE}"
else
  echo "FAILURE: Java executable not found in JAVA_HOME, JAVA_HOME path is invalid." > "${LOG_FILE}"
  echo "To set JAVA_HOME for the stream, run the following script: " >> "${LOG_FILE}" 
  echo "<ora-kafka-home>/orakafka/bin/orakafka.sh set_java_home -p <java-home-path>" >> "${LOG_FILE}"
  echo "where <java-home-path> should be a valid JDK/JRE installation directory" >> "${LOG_FILE}"
  echo "and the java version should be 1.8 or higher." >> "${LOG_FILE}"
  exit 1
fi

#
# The *.pp.bco file is populated by the preprocessor in the java code
# On non_RAC systems the listener process runs as "oracle" 
# on RAC systems the listener processor runs as "grid" 
# The listener process launches the preprocessor
# On RAC/DBCS environments "grid" user owns the *.pp.bco files
# Give group read/write permissions to .bco file 
# so this file can be deleted by "oracle" user from pl/sql
# using UTL_FILE functions
# we have touch the file and set permissions before the exec 
# before the existing shell is replaced.
BCO_FILE="${LOC_FILE}".pp.bco
touch "${BCO_FILE}"
chmod g+rw "${BCO_FILE}"

if [[ "$ENABLE_SSL_DEBUG" == "true" ]]; then
 ENABLE_SSL_DEBUG_ARG="-Djavax.net.debug=all"
fi

JVM_PROPS="-Dorakafka.conf.dir="${EXECUTE_DIR}/conf" \
  -Djava.library.path="${PRODUCT_KIT_HOME}/lib/" \
  -Dorg.xerial.snappy.use.systemlib=true \
  ${ENABLE_SSL_DEBUG_ARG}"

exec ${JAVA_HOME}/bin/java \
  -cp "${PRODUCT_KIT_HOME}/jlib/*" \
  ${JVM_PROPS} \
  oracle.hadoop.dbmskafka.KafkaStream "${LOC_FILE}" \
  3>&1 4>&2 5>"${LOG_FILE}" 1>/dev/null 2>&5

