#!/bin/bash
#
# add_cluster.sh
#
# Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      add_cluster.sh - Configures the Kafka cluster configuration
#                       filesystem directory and also assists in
#                       creation of the cluster configuration 
#                       database directory.
#
#    DESCRIPTION
#      Configures the Kafka cluster configuration filesystem directory
#      structure. 
#      This script creates the OS cluster configuration directory
#      for the Kafka cluster with the given cluster name. For
#      a secure cluster, the security properties need to be configured 
#      by the user in conf/orakafka.properties under the cluster
#      configuration directory.
#
#      In addition, it also assists in the creation of the cluster
#      configuration database directory.
# 
#      Usage: ./add_cluster.sh -c <cluster-name> [-h]
#             where options are:
#             -c <cluster-name> - Name of cluster directory to be
#                                 created (case-insensitive).    
#             -h                - Display usage
#
#      User model: This script must be run by the "oracle" user.
#
#      Additional options: (not exposed in the usage)
#         -s : This disables displaying the individual step summary.
#              It is used by the setup_all script which runs
#              multiple steps and displays a summary at the end.
#
#    NOTES
#      For debugging, set environment variable ORAKAFKA_CONFIG_DEBUG to true.
#      export ORAKAFKA_CONFIG_DEBUG=true
# 

set -e

SCRIPT_NAME=$(basename $0)

# Prints usage to the terminal and exits
function usage() {
  echo "Usage: ${SCRIPT_NAME} -c <cluster-name> [-h]"
  echo "where options are:"
  echo "-c <cluster-name> - Name of cluster directory to be created"
  echo "                    (case-insensitive)"
  echo "-h                - Display usage"
  exit 1
}

function cleanup {
  printstepmsg "$STEP failed."
  CLUSTER_DIR_PATH=$1
  if [[ -d "${CLUSTER_DIR_PATH}" ]]; then
    echo "Removing the cluster filesystem directory \"${CLUSTER_DIR_PATH}\""
    rm -rf "${CLUSTER_DIR_PATH}";
  fi
  exit 1
}

function create_cluster_fsdir {
 trap "cleanup ${CLUSTER_DIR_PATH}" INT ERR TERM
 # create the filesystem cluster conf directory
 printheader "$STEP: Creation of filesystem cluster config directory"
 if [[ -d "${CLUSTER_DIR_PATH}" ]]; then
  printstepmsg "FAILURE: Specified directory name \"${CLUSTER_DIR_NAME}\" already exists."
  exit 1
 else
  mkdir "${CLUSTER_DIR_PATH}"
  debug "Created cluster directory: ${CLUSTER_DIR_PATH}"
  cp ${ORAKAFKA_BIN_DIR}/orakafka_stream.sh "${CLUSTER_DIR_PATH}"
  debug "Copied ${ORAKAFKA_BIN_DIR}/orakafka_stream.sh to ${CLUSTER_DIR_PATH}"
  mkdir "${CLUSTER_CONF_DIR_PATH}"
  debug "Created configuration directory for cluster: ${CLUSTER_CONF_DIR_PATH}"
  cp ${ORAKAFKA_CONF_DIR}/orakafka.properties.template "${CLUSTER_CONF_DIR_PATH}"/orakafka.properties
  debug "Copied ${ORAKAFKA_CONF_DIR}/orakafka.properties.template to ${CLUSTER_CONF_DIR_PATH}"
  mkdir "${CLUSTER_LOG_DIR_PATH}"
  # group permissions for the cluster configuration directory
  # in case of RAC, the pre-processor runs as the grid user, therefore,
  # in addition to owner "oracle" user, the "grid" user also needs to be
  # able to read and execute the stream script.
  chmod -R 750 "${CLUSTER_DIR_PATH}"
  chmod 640 "${CLUSTER_CONF_DIR_PATH}"/orakafka.properties
  printstepmsg "Filesystem cluster directory creation completed."
  printstepmsg "Configure security properties at ${CLUSTER_CONF_DIR_PATH}/orakafka.properties."
 fi
}

function generateddl_create_cluster_dir {
 trap "cleanup ${CLUSTER_DIR_PATH}" INT ERR TERM
 printheader "$STEP: Generate DDL for creation of cluster config DB directory"
cat > "${CLUSTER_CONF_DB_SCRIPT}" <<EOF
--generated by ORAKAFKA config CLI - add_cluster.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
spool ${CLUSTER_CONF_DB_LOG};
prompt Creating database directory "${CLUSTER_CONF_DB_DIR}"..
CREATE DIRECTORY "${CLUSTER_CONF_DB_DIR}" as '${CLUSTER_DIR_PATH}';
prompt The above information is written to ${CLUSTER_CONF_DB_LOG}
spool off;
EOF
if [[ -z ${DISABLE_SUMMARY} ]]; then
 cat >> "${CLUSTER_CONF_DB_SCRIPT}" <<EOF
exit;
EOF
fi

 printstepmsg "Execute the following SQL script while connected as sysdba"
 printstepmsg "to create cluster config database directory:"
 printstepmsg " @${CLUSTER_CONF_DB_SCRIPT}"
 chmod 755 "${CLUSTER_CONF_DB_SCRIPT}"
}


source $(dirname $0)/config_util.sh

# if no arguments are passed, then print usage
if [[ $# -eq 0 ]]; then
  usage
fi

debug "Arguments passed to ${SCRIPT_NAME}: $*"

while getopts ":c:sh" opt;
do
  case "$opt" in
    c) CLUSTER_NAME=$OPTARG ;;
    s) DISABLE_SUMMARY=1;;
    h) usage ;;
   \?) echo "Invalid option -$OPTARG"
       usage ;;
    :) echo "Option -$OPTARG requires an argument."
       usage ;;
  esac
done
# after getopts is done, shifts away all the processed options.
shift "$(($OPTIND -1))"

CLUSTER_DIR_NAME=$(validateclustername "${CLUSTER_NAME}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

debug "CLUSTER_DIR_NAME=\"${CLUSTER_DIR_NAME}\""
debug "APP_DATA_CLUSTERS_DIR=${APP_DATA_CLUSTERS_DIR}"
debug "PRODUCT_HOME=${PRODUCT_HOME}"
debug "KIT_HOME=${ORAKAFKA_HOME}"

# Create the filesystem OS directories in the cluster directory
# Filesystem cluster conf directory structure
# clusters/
#  cluster1/
#    orakafka_stream.sh
#    conf/
#      orakafka.properties
#    logs/

CLUSTER_DIR_PATH="${APP_DATA_CLUSTERS_DIR}/${CLUSTER_DIR_NAME}"
CLUSTER_CONF_DIR_PATH="${CLUSTER_DIR_PATH}/conf"
CLUSTER_LOG_DIR_PATH="${CLUSTER_DIR_PATH}/logs"

trap "cleanup ${CLUSTER_DIR_PATH}" INT ERR TERM

STEP="Step1"
create_cluster_fsdir
printstatus "$?" "$STEP"

# generate DDL for the creation of the cluster configuration DB directory.
CLUSTER_CONF_DB_DIR="${CLUSTER_DIR_NAME}"_CONF_DIR
debug "CLUSTER_CONF_DB_DIR=${CLUSTER_CONF_DB_DIR}"
CLUSTER_CONF_DB_SCRIPT="${APP_DATA_SCRATCH_DIR}/orakafka_create_"${CLUSTER_CONF_DB_DIR}".sql"
CLUSTER_CONF_DB_LOG="${APP_DATA_LOGS_DIR}/orakafka_create_"${CLUSTER_CONF_DB_DIR}".log"

STEP="Step2"
generateddl_create_cluster_dir
printgenscriptstatus "$?" "$STEP"

if [[ -z "$DISABLE_SUMMARY" ]]; then
 echo -e "\n***********SUMMARY************"
 echo -e "\nTODO tasks:\n"
 echo "1. Configure security properties at ${CLUSTER_CONF_DIR_PATH}/orakafka.properties"
 echo "2. Execute the following SQL while connected as sysdba: "
 echo "   @${CLUSTER_CONF_DB_SCRIPT}"
fi
