#!/bin/bash
#
# adduser_cluster.sh
#
# Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      adduser_cluster.sh
#
#    DESCRIPTION
#      Grants permissions on a particular cluster to a DB user.
#      It generates the DDL that grants privileges on the cluster
#      configuration directory to the given database user.
#
#      Usage: ./adduser_cluster.sh -c <cluster-name> -u <db-user> [-h]
#             where options are:
#             -c <cluster-name> - Kafka cluster name for which DB user needs
#                                 access (case-insensitive)
#             -u <db-user>      - DB username (case-insensitive)
#             -h                - Display usage
#
#      Additional options: (not exposed in the usage)
#         -i <userid> : Used to pass the userid generated for the DB username.
#                       This option is used by the setup_all script.
#         -s : This disables displaying the individual step summary.
#              It is used by the setup_all script which runs
#              multiple steps and displays a summary at the end.

#
#    NOTES
#      For debugging, set enviroment variable ORAKAFKA_CONFIG_DEBUG to true.
#      export ORAKAFKA_CONFIG_DEBUG=true
#

SCRIPT_NAME=$(basename $0)

function usage {
 echo "Usage: ${SCRIPT_NAME} -c <cluster-name> -u <db-user> [-h]"
 echo "where options are:"
 echo "-c <cluster-name> - Kafka cluster name for which the DB user"
 echo "                    needs access (case-insensitive)"
 echo "-u <db-user>      - DB username (case-insensitive)"
 echo "-h                - Display usage"
 exit 1
}

function failstep {
 echo "$STEP failed."
 exit 1
}

function adduser_cluster {
 trap "failstep" INT ERR TERM
 printheader "$STEP: Generate DDL to grant permissions on cluster configuration directory to \"${DB_USER}\""
 cat > "${ADDUSER_CLUSTER_SCRIPT}" <<EOF
--generated by ORAKAFKA CLI - adduser_cluster.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
spool ${ADDUSER_CLUSTER_LOG};
DECLARE
 v_clusterDirPath varchar2(4000);
BEGIN
 select directory_path into v_clusterDirPath from sys.all_directories where directory_name='${CLUSTER_CONF_DB_DIR}';
EXCEPTION
 WHEN NO_DATA_FOUND THEN
   raise_application_error(-20000, 'Directory ${CLUSTER_CONF_DB_DIR} does not exist.');
 WHEN OTHERS THEN 
   RAISE;
 END;
/
prompt Granting permissions on "${CLUSTER_CONF_DB_DIR}" to "${DB_USER}"
GRANT READ,EXECUTE ON DIRECTORY "${CLUSTER_CONF_DB_DIR}" to "${DB_USER}";
prompt The above information is written to ${ADDUSER_CLUSTER_LOG}
spool off;
EOF
if [[ -z ${DISABLE_SUMMARY} ]]; then
 cat >> "${ADDUSER_CLUSTER_SCRIPT}" <<EOF
exit;
EOF
fi

 printstepmsg "Execute the following script while connected as sysdba"
 printstepmsg "to grant permissions on cluster conf directory : "
 printstepmsg " @${ADDUSER_CLUSTER_SCRIPT}"
 chmod 755 "${ADDUSER_CLUSTER_SCRIPT}"
}

source $(dirname $0)/config_util.sh

# if no arguments are passed, then print usage
if [[ $# -eq 0 ]]; then
  usage
fi

debug "Arguments passed to ${SCRIPT_NAME}: $*"

while getopts ":u:c:i:sh" opt;
do
  case "$opt" in
  c) CLUSTER_NAME=$OPTARG;;
  u) DB_USER=$OPTARG;;
  i) USERID=$OPTARG;;
  s) DISABLE_SUMMARY=1;;
  h) usage;;
 \?) echo "Invalid option -$OPTARG"
     usage ;;
  :) echo "Option -$OPTARG requires an argument."
     usage ;;
  esac
done

CLUSTER_DIR_NAME=$(validateclustername "${CLUSTER_NAME}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

# check if cluster fs directory exists
if [[ ! -d "${APP_DATA_CLUSTERS_DIR}/${CLUSTER_DIR_NAME}" ]]; then
 failuremsg "Specified cluster directory ${CLUSTER_DIR_NAME} does not exist."
 echo "Please run \"./orakafka.sh add_cluster -c \"$CLUSTER_DIR_NAME\"\" to create the cluster directory."
 exit 1
fi

#validate db user
DB_USER=$(validatedbuser "${DB_USER}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

# validate and get userid
USERID=$(validateandgetuserid "${USERID}" "${DB_USER}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

# name of cluster DB directory
CLUSTER_CONF_DB_DIR="${CLUSTER_DIR_NAME}"_CONF_DIR

ADDUSER_CLUSTER_SCRIPT=${APP_DATA_SCRATCH_DIR}/orakafka_adduser_cluster_${CLUSTER_DIR_NAME}_${USERID}.sql
ADDUSER_CLUSTER_LOG=${APP_DATA_LOGS_DIR}/orakafka_adduser_cluster_${CLUSTER_DIR_NAME}_${USERID}.log

STEP="Step1"
adduser_cluster
printgenscriptstatus "$?" "$STEP"

# summary
if [[ -z "$DISABLE_SUMMARY" ]]; then
 echo -e "\n***********SUMMARY************"
 echo -e "\nTODO tasks:\n"
 echo "1. Execute the following SQL while connected as sysdba: "
 echo "   @${ADDUSER_CLUSTER_SCRIPT}"
fi
