#!/bin/bash
#
# config_util.sh
#
# Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      config_util.sh
#
#    DESCRIPTION
#      Utility functions for configuration of Oracle SQL Access to Kafka.
#
#    NOTES
#      
#
#

function debug {
  if [[ "${ORAKAFKA_CONFIG_DEBUG}" == true ]]; then
    # if a debug flag is set, echo the message
    if [[ ! -z "$1" ]]; then
      echo "DEBUG: $1"
    fi
  fi
}

# Discovers ORAKAFKA_HOME
# Reference:
# http://stackoverflow.com/questions/59895/can-a-bash-script-tell-what-directory-its-stored-in
SOURCE="${BASH_SOURCE[0]}"
while [ -h "${SOURCE}" ] # resolve source until the file is no longer a symlink
do
  BIN_SCRIPTS_DIR="$( cd -P "$( dirname "$SOURCE" )" >/dev/null 2>&1 && pwd )"
  SOURCE="$(readlink "${SOURCE}")"
  # if $SOURCE is a relative symlink, resolve it relative to path
  # where the symlink file is located.
  [[ ${SOURCE} != /* ]] && SOURCE="${BIN_SCRIPTS_DIR}/${SOURCE}"
done
BIN_SCRIPTS_DIR="$( cd -P "$( dirname "${SOURCE}" )" >/dev/null 2>&1 && pwd )"
ORAKAFKA_BIN_DIR="$( dirname "${BIN_SCRIPTS_DIR}")"
ORAKAFKA_HOME="$( dirname "${ORAKAFKA_BIN_DIR}")"
PRODUCT_HOME="$( dirname "${ORAKAFKA_HOME}")"

debug "PRODUCT_HOME=\"${PRODUCT_HOME}\""
debug "ORAKAFKA_HOME=\"${ORAKAFKA_HOME}\""
debug "ORAKAFKA_BIN_DIR=\"${ORAKAFKA_BIN_DIR}\""


SCRIPT_CONFIG_APP_DATA_HOME="${PRODUCT_HOME}/configure_app_data_home.sh"

if [[ ! -f "${SCRIPT_CONFIG_APP_DATA_HOME}" ]]; then
   echo "script ${SCRIPT_CONFIG_APP_DATA_HOME} not found" 
   echo "This script is generated by orakafka_distro_install.sh"
   exit 1
fi   

source "${SCRIPT_CONFIG_APP_DATA_HOME}"
if [[ -z "${APP_DATA_HOME}" ]]; then
  echo "APP_DATA_HOME not found in ${SCRIPT_CONFIG_APP_DATA_HOME}"
  echo "This value is generated by orakafka_distro_install.sh"
  exit 1
fi

if [[ ! -d "${APP_DATA_HOME}" ]]; then
   echo "APP_DATA_HOME: [${APP_DATA_HOME}] directory does not exist"
   echo "This directory is created by orakafka_distro_install.sh"
   exit 1
fi  

APP_DATA_SCRATCH_DIR="${APP_DATA_HOME}"/scratch
APP_DATA_LOGS_DIR="${APP_DATA_HOME}"/logs
APP_DATA_CLUSTERS_DIR="${APP_DATA_HOME}"/clusters
APP_DATA_SCRIPTS_DIR="${APP_DATA_HOME}"/scripts
ORAKAFKA_CONF_DIR="${ORAKAFKA_HOME}"/conf
ORAKAFKA_SQL_DIR="${ORAKAFKA_HOME}"/sql

debug "APP_DATA_SCRATCH_DIR=\"${APP_DATA_SCRATCH_DIR}\""
debug "APP_DATA_LOGS_DIR=\"${APP_DATA_LOGS_DIR}\""
debug "APP_DATA_CLUSTERS_DIR=\"${APP_DATA_CLUSTERS_DIR}\""
debug "APP_DATA_SCRIPTS_DIR=\"${APP_DATA_SCRIPTS_DIR}\""
debug "ORAKAFKA_CONF_DIR=\"${ORAKAFKA_CONF_DIR}\""
debug "ORAKAFKA_SQL_DIR=\"${ORAKAFKA_SQL_DIR}\""

function printstepmsg {
  MSG="$1"  
  printf "%s\n" "$MSG"
}

function printstatus {
  ret_code=$1
  step=$2
  if [[ "$ret_code" -eq 0 ]]; then
    echo "$step succeeded."
  fi
}

function printgenscriptstatus {
  ret_code=$1
  step=$2
  if [[ "$ret_code" -eq 0 ]]; then
    echo "$step successfully generated script."
  fi
}

function failuremsg {
  printf "FAILURE: %s\n" "$1" >&2
}

function successmsg {
  printf "SUCCESS: %s\n" "$1"
}

function printheader {
  printf "\n%s\n" "$1"
  printf "%s\n" "--------------------------------------------------------------"
}


function getuserid {
 DB_USER="$1"
 # if usermap.txt doesn't exist, create it
 # map user to a user<num> name
 if [[ ! -f "${APP_DATA_SCRATCH_DIR}/usermap.txt" ]]; then
   echo "\"${DB_USER}\"=\"user1\"" >> ${APP_DATA_SCRATCH_DIR}/usermap.txt
   echo "user1"
   return 0
 else
   # read from file, if entry found use the value
   # otherwise, create a new entry
   keys=()
   values=()
   while IFS="=" read -r key value
   do
     keys+=("$key")
     values+=("$value")
   done < ${APP_DATA_SCRATCH_DIR}/usermap.txt
   for i in "${!keys[@]}"; do
     if [[ ${keys[i]} = \"${DB_USER}\" ]]; then
       echo ${values[i]} | tr -d '"'
       return 0
     fi
   done
   # if user not found, get last user
   lastuser=$(tail -n 1 ${APP_DATA_SCRATCH_DIR}/usermap.txt)
   lastuserid=$(awk -F= '{print $2}'<<< "$lastuser")   
   lastid="${lastuserid: -2:1}"
   nextid=$((lastid+1))  
   echo "\"${DB_USER}\"=\"user${nextid}\"" >> ${APP_DATA_SCRATCH_DIR}/usermap.txt
   echo "user${nextid}"
   return 0
 fi
}


#The arguments must be an unquoted db user name with the additional
#restrictions that hash (#) and dollar sign ($) are not allowed.
#
#I.e., it must start with an alphabetic character, and contain
#only alphanumeric characters and underscore (_).
#
#Quotes(") are not allowed.
#Lower case chars are allowed, and will be uppercased.
#
function validatedbuser {
 DB_USER="$1"
 if ! [[ ${DB_USER} =~ ^[[:alpha:]]([[:alnum:]]|_)*$ ]]
 then
  local msg="DB user name must start with an alphabetic character and can"
  msg+=" contain only alphanumeric characters and underscore;"
  msg+=" given user name=${DB_USER}"
  failuremsg "$msg"
  return 1  
 fi
 DB_USER=${DB_USER^^}
 echo "${DB_USER}"
 return 0
}



function validateclustername {
 CLUSTER_NAME="$1"
 # trims any leading or trailing spaces in the given cluster name
 CLUSTER_DIR_NAME=$(printf "$CLUSTER_NAME" |  sed -e 's/^[[:space:]]*//;s/[[:space:]]*$//')

 if [[ -z "${CLUSTER_DIR_NAME}" ]]; then
  failuremsg "Cluster name is empty or invalid."
  return 1
 fi

 # cluster name can only consist of alphanumeric chars
 CLUSTER_DIR_NAME_ALPHANUM="$(echo $CLUSTER_DIR_NAME | sed -e 's/[^[:alnum:]]//g')"

 if [[ $CLUSTER_DIR_NAME != $CLUSTER_DIR_NAME_ALPHANUM ]]; then
  failuremsg "Only alphanumeric characters allowed in cluster name, given clustername=$CLUSTER_DIR_NAME"
  return 1
 fi

 echo "${CLUSTER_DIR_NAME^^}"
 return 0
}

# Assumes that the second argument (DB_USER) has already been validated.
#
#calling validatedbuser both inside validateandgetuserid
#and at call site generates confusing double failure printouts
function validateandgetuserid {
 USERID="$1"
 DB_USER="$2"
 if [[ -z "$USERID" ]]; then
  if [[ -z "$DB_USER" ]]; then
   failuremsg "Empty or invalid DB username."
   return 1
  fi
  USERID=$(getuserid "${DB_USER}")
  if [[ -z "$USERID" ]]; then
   failuremsg "Empty or invalid user id."
   return 1
  fi
 fi
 echo "${USERID}"
 return 0
}

