#!/bin/bash
#
# install.sh
#
# Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      install.sh
#
#    DESCRIPTION
#      Installs ORA_KAFKA package for a particular DB user.
#      This generates DDL for creation of the database location
#      and default directories for the given user, 
#      grants necessary permissions (READ, WRITE) on the
#      default and location directories
#      and also generates DDL to install ORA_KAFKA package 
#      in the user schema.
#
#      Usage: ./install.sh -u <db-user> -r <root-location> [-h]
#             where options are:
#             -u <db-user>       - User schema for which ORA_KAFKA package
#                                  is to be setup (case-insensitive)
#             -r <root-location> - Filesystem location under which location
#                                  and default dirs are created
#             -h                 - Display usage
#
#      Additional options: (not exposed in the usage)
#         -i <userid> : Used to pass the userid generated for the DB username.
#                       This option is used by the setup_all script.
#         -s : This disables displaying the individual step summary.
#              It is used by the setup_all script which runs
#              multiple steps and displays a summary at the end.
#
#    NOTES
#      <other useful comments, qualifications, etc.>
#

SCRIPT_NAME=$(basename $0)

function usage {
  echo -e "Usage: ${SCRIPT_NAME} -u <db-user> -r <root-location> [-h]"
  echo "where options are:"
  echo "-u <db-user>       - User schema for which ORA_KAFKA package is to"
  echo "                     be setup (case-insensitive)"
  echo "-r <root-location> - Filesystem location under which location and default dirs are created"
  echo "-h                 - Display usage"
  exit 1
}

function cleanup {
  echo "$STEP failed."
  ROOT_LOCATION="$1"
  echo "Cleaning up loc and default directories in ${ROOT_LOCATION}"
  if [[ -d "${ROOT_LOCATION}/orakafka_location_dir" ]]; then
    rm -rf "${ROOT_LOCATION}/orakafka_location_dir";
  fi
  if [[ -d "${ROOT_LOCATION}/orakafka_default_dir" ]]; then
    rm -rf "${ROOT_LOCATION}/orakafka_default_dir";
  fi
  exit 1
}

function create_locdef_fsdirs {
 trap "cleanup ${ROOT_LOCATION}" INT ERR TERM
 printheader "$STEP: Creation of filesystem location and default directories"
 if [ -d "${ROOT_LOCATION}/orakafka_location_dir" ] || [ -d "${ROOT_LOCATION}/orakafka_default_dir" ]; then
  printstepmsg "FAILURE: Location directory and default directory already exist under ${ROOT_LOCATION}"
  exit 1
 else
  LOC_DIR_PATH="${ROOT_LOCATION}/orakafka_location_dir"
  DEF_DIR_PATH="${ROOT_LOCATION}/orakafka_default_dir"
  mkdir "${LOC_DIR_PATH}"
  chmod -R 770 ${LOC_DIR_PATH}
  mkdir "${DEF_DIR_PATH}"
  chmod -R 700 ${DEF_DIR_PATH}
  printstepmsg "Created filesystem location directory at ${LOC_DIR_PATH}"
  printstepmsg "Created filesystem default directory at ${DEF_DIR_PATH}"
  REMOVE_FSDIRS_SCRIPT="${APP_DATA_SCRATCH_DIR}/remove_fsdirs_${USERID}.sh"
  cat > "${REMOVE_FSDIRS_SCRIPT}" <<EOF
#!/bin/bash
#generated by ORAKAFKA CLI - install.sh
LOC_DIR_DEL_SUC=""
DEF_DIR_DEL_SUC=""
if [[ -d "$LOC_DIR_PATH" ]]; then
 rm -rf "$LOC_DIR_PATH"
 LOC_DIR_DEL_SUC="\$?"
 if [[ "\$LOC_DIR_DEL_SUC" -eq 0 ]]; then
  echo "Removed location directory $LOC_DIR_PATH"
 fi
else
 echo "Location directory $LOC_DIR_PATH does not exist."
fi
if [[ -d "$DEF_DIR_PATH" ]]; then
 rm -rf "$DEF_DIR_PATH"
 DEF_DIR_DEL_SUC="\$?"
 if [[ "\$DEF_DIR_DEL_SUC" -eq 0 ]]; then
   echo "Removed default directory $DEF_DIR_PATH"
 fi
else
 echo "Default directory $DEF_DIR_PATH does not exist."
fi
if [[ "\$LOC_DIR_DEL_SUC" -eq 0  && "\$DEF_DIR_DEL_SUC" -eq 0 ]]; then
  exit 0
else
  exit 1
fi
EOF
  chmod 755 "${REMOVE_FSDIRS_SCRIPT}"
 fi    
}

function generateddl_create_locdef_db_dirs {
 trap "cleanup ${ROOT_LOCATION}" INT ERR TERM
 printheader "$STEP: Generate DDL for creation of DB location and default directories"
   CHECK_USER_EXISTS="-- check if the user exists
DECLARE
v_userValue varchar2(200) ;
BEGIN
 select sys.dbms_assert.schema_name('${DB_USER}') into v_userValue from sys.dual;
EXCEPTION 
WHEN sys.dbms_assert.INVALID_SCHEMA_NAME THEN
  raise_application_error(-20001, 'User ${DB_USER} does not exist');
END;
/"
  SETUP_DB_DIRS="--check if the directory already exists
DECLARE
v_locDirPath varchar2(4000);
v_defDirPath varchar2(4000);
BEGIN
 BEGIN
 select directory_path into v_locDirPath from sys.all_directories where directory_name='${LOC_DB_DIR}';
 sys.dbms_output.put_line('${LOC_DB_DIR} already exists.');
 sys.dbms_output.put_line('${LOC_DB_DIR} path = ' || v_locDirPath);
 IF v_locDirPath <> '${LOC_DIR_PATH}' THEN
  raise_application_error(-20000, 'Existing ${LOC_DB_DIR} does not match ${LOC_DIR_PATH}');
 END IF;
 EXCEPTION
  WHEN NO_DATA_FOUND THEN NULL;
 END;
 BEGIN
 select directory_path into v_defDirPath from sys.all_directories where directory_name='${DEF_DB_DIR}';
 sys.dbms_output.put_line('${DEF_DB_DIR} already exists.');
 sys.dbms_output.put_line('${DEF_DB_DIR} path = ' || v_defDirPath);
 IF v_defDirPath <> '${DEF_DIR_PATH}' THEN
  raise_application_error(-20000, 'Existing ${DEF_DB_DIR} does not match ${DEF_DIR_PATH}');
 END IF;
 EXCEPTION
  WHEN NO_DATA_FOUND THEN NULL;
 END;
END;
/
 CREATE OR REPLACE DIRECTORY \"${LOC_DB_DIR}\" as '${LOC_DIR_PATH}';
 CREATE OR REPLACE DIRECTORY \"${DEF_DB_DIR}\" as '${DEF_DIR_PATH}';
 GRANT READ, WRITE ON DIRECTORY \"${LOC_DB_DIR}\" to \"${DB_USER}\";
 GRANT READ, WRITE ON DIRECTORY \"${DEF_DB_DIR}\" to \"${DB_USER}\";
"
cat > "${SETUP_DB_DIRS_SCRIPT}" <<EOF
--generated by ORAKAFKA CLI - install.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
spool ${SETUP_DB_DIRS_LOG};
prompt Checking if user exists..
${CHECK_USER_EXISTS}
prompt Creating location and default directories..
${SETUP_DB_DIRS}
prompt Creation of location dir "${LOC_DB_DIR}" and default dir "${DEF_DB_DIR}" completed.
prompt Grant of required permissions on "${LOC_DB_DIR}","${DEF_DB_DIR}" to "${DB_USER}" completed.
prompt The above information is written to ${SETUP_DB_DIRS_LOG}
spool off;
EOF
if [[ -z ${DISABLE_SUMMARY} ]]; then
 cat >> ${SETUP_DB_DIRS_SCRIPT} <<EOF
exit;
EOF
fi

 printstepmsg "Execute the following SQL script while connected as sysdba"
 printstepmsg "to setup database directories: "
 printstepmsg " @${SETUP_DB_DIRS_SCRIPT}"
 printstepmsg "On failure, to cleanup location and default directory setup, please run \"./orakafka.sh uninstall -u '${DB_USER}'\""
 chmod 755 "${SETUP_DB_DIRS_SCRIPT}"
}

function install_orakafka_user {
 trap "cleanup ${ROOT_LOCATION}" INT ERR TERM
 printheader "$STEP: Install ORA_KAFKA package in \"${DB_USER}\" user schema"
cat > "${INSTALL_ORAKAFKA_SCRIPT}" <<EOF
--generated by ORAKAFKA config CLI - install.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
spool ${INSTALL_ORAKAFKA_LOG};
prompt Verifying user schema..
DECLARE
 v_valid_schema NUMBER;
BEGIN
 select 1 into v_valid_schema from sys.dual where SYS_CONTEXT('USERENV', 'CURRENT_SCHEMA')='${DB_USER}';
EXCEPTION
 WHEN NO_DATA_FOUND THEN
   raise_application_error(-20000, 'Current schema is not "${DB_USER}".');
END;
/
--verify that the loc and default directories are accessible.
prompt Verifying that location and default directories are accessible..
DECLARE
v_acc_dirs NUMBER;
BEGIN
 select count(1) into v_acc_dirs from sys.all_directories where directory_name IN ('${LOC_DB_DIR}','${DEF_DB_DIR}');
 IF v_acc_dirs <> 2 THEN
   raise_application_error(-20000, 'Location and default directories ${LOC_DB_DIR},${DEF_DB_DIR} not accessible by user');
 END IF;
END;
/
prompt Installing ORA_KAFKA package in user schema..
@${ORAKAFKA_SQL_DIR}/catorakafka.sql
prompt The above information is written to ${INSTALL_ORAKAFKA_LOG}
spool off;
exit;
EOF
 printstepmsg "Execute the following script in user schema \"${DB_USER}\" to"
 printstepmsg "install ORA_KAFKA package in the user schema"
 printstepmsg " @${INSTALL_ORAKAFKA_SCRIPT}"
 printstepmsg "On failure, to cleanup ORA_KAFKA package from user schema, please run \"./orakafka.sh uninstall -u '${DB_USER}'\""
 chmod 755 "${INSTALL_ORAKAFKA_SCRIPT}"
}

source $(dirname $0)/config_util.sh

if [[ $# -eq 0 ]]; then
  usage
fi

debug "Arguments passed to ${SCRIPT_NAME}: $*"

while getopts ":u:r:i:sh" opt;
do
 case "$opt" in
  u) DB_USER=$OPTARG;;
  r) ROOT_LOCATION=$OPTARG
     if [[ ! -d "$ROOT_LOCATION" ]]; then
       failuremsg "Invalid root location provided."
       exit 1
     else
       # if root location already exists, check
       # if loc and def dir are already present.
       # if yes, fail
       if [ -d "$ROOT_LOCATION/orakafka_location_dir" ] || [ -d "$ROOT_LOCATION/orakafka_default_dir" ]; then
         failuremsg "Location or default directory already present in $ROOT_LOCATION."
	 exit 1
       fi
     fi;;
  s) DISABLE_SUMMARY=1;;
  i) USERID=$OPTARG;;
  h) usage;;
 \?) echo "Invalid option -$OPTARG"
     usage ;;
  :) echo "Option -$OPTARG requires an argument."
     usage ;;
 esac
done

#validate db user
DB_USER=$(validatedbuser "${DB_USER}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

# validate and get userid
USERID=$(validateandgetuserid "${USERID}" "${DB_USER}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

debug "USERID=${USERID}"

if [[ -z "$ROOT_LOCATION" ]]; then
  failuremsg "Empty or invalid root location provided."
  usage
fi

# setup dirs
SETUP_DB_DIRS_SCRIPT=${APP_DATA_SCRATCH_DIR}/setup_db_dirs_${USERID}.sql
SETUP_DB_DIRS_LOG=${APP_DATA_LOGS_DIR}/setup_db_dirs_${USERID}.log

# create the location and default directories
# under the root location
STEP="Step1"
create_locdef_fsdirs
printstatus "$?" "$STEP"

# generate DDL to create the corresponding DB directories and grant read, write 
# permissions on those directories to the given DB user.
LOC_DB_DIR="${DB_USER}"_KAFKA_LOC_DIR
DEF_DB_DIR="${DB_USER}"_KAFKA_DEF_DIR

STEP="Step2"
generateddl_create_locdef_db_dirs
printgenscriptstatus "$?" "$STEP"

INSTALL_ORAKAFKA_SCRIPT=${APP_DATA_SCRATCH_DIR}/install_orakafka_${USERID}.sql  
INSTALL_ORAKAFKA_LOG=${APP_DATA_LOGS_DIR}/install_orakafka_${USERID}.log

STEP="Step3"
install_orakafka_user
printgenscriptstatus "$?" "$STEP"

if [[ -z "$DISABLE_SUMMARY" ]]; then
 echo -e "\n***********SUMMARY************"
 echo -e "\nTODO tasks:\n"
 echo "1. Execute the following SQL while connected as sysdba: "
 echo "   @${SETUP_DB_DIRS_SCRIPT}"
 echo "2. Execute the following SQL in user schema: "
 echo "   @${INSTALL_ORAKAFKA_SCRIPT}"
fi
