#!/bin/bash
#
# list_clusters.sh
#
# Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      list_clusters.sh
#
#    DESCRIPTION
#      Lists all the clusters that have been configured for Oracle SQL
#      Access to Kafka.
#      This command queries the all_directories view in the database
#      to retrieve the DB cluster configuration directories and their
#      corresponding filesystem directories.
#
#      Usage: ./list_clusters.sh [-h]
#             where options are:
#             -h                - Display usage
#
#      Additional options: (not exposed in the usage)
#         -s : This disables displaying the individual step summary.
#              It is used by the setup_all script which runs
#              multiple steps and displays a summary at the end.
#
#    NOTES
#      <other useful comments, qualifications, etc.>
#

SCRIPT_NAME=$(basename $0)

function usage {
  echo "Usage: ${SCRIPT_NAME} [-h]"
  echo "where options are:"
  echo "-h             - Display usage"
  exit 1
}

function failstep {
  echo "$STEP failed."
  exit 1
}

function generateddl_listclusters {
 trap "failstep" INT ERR TERM
 printheader "$STEP: Generate DDL to list all clusters"
cat > "${LIST_ALL_CLUSTERS_SCRIPT}" <<EOF
--generated by ORAKAFKA CLI - list_clusters.sh
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
spool ${LIST_ALL_CLUSTERS_LOG};
select substr(directory_name,1,length(directory_name)-9) as
cluster_name,directory_name, directory_path from sys.all_directories where directory_name IN ${IN_DIR_LIST}
order by cluster_name;
spool off;
exit;
EOF
 printstepmsg "Execute the following SQL script while connected as sysdba"
 printstepmsg "to list all clusters: "
 printstepmsg " @${LIST_ALL_CLUSTERS_SCRIPT}"
 chmod 755 "${LIST_ALL_CLUSTERS_SCRIPT}"
}

source $(dirname $0)/config_util.sh

while getopts ":sh" opt;
do
  case "$opt" in
    s) DISABLE_SUMMARY=1;;
    h) usage;;
   \?) echo "Invalid option -$OPTARG"
       usage ;;
    :) echo "Option -$OPTARG requires an argument."
       usage ;;
  esac
done
# after getopts is done, shifts away all the processed options.
shift "$(($OPTIND -1))"

LIST_ALL_CLUSTERS_SCRIPT="${APP_DATA_SCRATCH_DIR}/list_all_clusters.sql"
LIST_ALL_CLUSTERS_LOG="${APP_DATA_LOGS_DIR}/list_all_clusters.log"

COUNTER=0
ALL_CONF_DIRS=""
# generate IN list for all cluster config DB directories
for dir in ${APP_DATA_CLUSTERS_DIR}/*; do
  if [[ -d "$dir" && ! -L "$dir" ]]; then
    DIR_NAME=$(basename $dir)
    if [[ $COUNTER -gt 0 ]]; then
     ALL_CONF_DIRS="$ALL_CONF_DIRS,"
    fi
    ALL_CONF_DIRS="$ALL_CONF_DIRS '${DIR_NAME}_CONF_DIR'"
    COUNTER=$((COUNTER+1))
  fi
done
if [[ -z "$ALL_CONF_DIRS" ]]; then
  failuremsg "No filesystem cluster directories found under ${APP_DATA_CLUSTERS_DIR}."
  exit 1
else
  IN_DIR_LIST="(${ALL_CONF_DIRS})"
fi

STEP="Step1"
generateddl_listclusters
printgenscriptstatus "$?" "$STEP"

# summary
if [[ -z "$DISABLE_SUMMARY" ]]; then
 echo -e "\n***********SUMMARY************"
 echo -e "\nTODO tasks:\n"
 echo "1. Execute the following SQL while connected as sysdba: "
 echo "   @${LIST_ALL_CLUSTERS_SCRIPT}"
fi
