#!/bin/bash
#
# remove_cluster.sh
#
# Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      remove_cluster.sh
#
#    DESCRIPTION
#      Removes the cluster configuration filesystem directory and
#      also assists in the dropping of the cluster configuration 
#      database directory.
#
#      Usage: ./remove_cluster.sh -c <cluster-name> [-h]
#             where options are:
#             -c <cluster-name> - Name of cluster directory to be
#                                 removed (case-insensitive)
#             -h                - Display usage
#
#      Additional options: (not exposed in the usage)
#         -s : This disables displaying the individual step summary.
#              It is used by the setup_all script which runs
#              multiple steps and displays a summary at the end.
#
#    NOTES
#      For debugging, set enviroment variable ORAKAFKA_CONFIG_DEBUG to true.
#      export ORAKAFKA_CONFIG_DEBUG=true
#

SCRIPT_NAME=$(basename $0)

function usage {
  echo "Usage: ${SCRIPT_NAME} -c <cluster-name> [-h]"
  echo "where options are:"
  echo "-c <cluster-name> - Name of cluster directory to be "
  echo "                    removed (case-insensitive)"
  echo "-h                - Display usage"
  exit 1
}

function failstep {
  printstepmsg "$STEP failed."
  exit 1
}

function generateddl_remove_dir {
  trap "failstep" INT ERR TERM
  printheader "$STEP: Generate DDL to remove cluster config DB directory"
  cat > "${CLUSTER_CONF_DB_SCRIPT}" <<EOF
--generated by ORAKAFKA CLI - remove_cluster.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
spool ${CLUSTER_CONF_DB_LOG};
-- checks if directory exists
-- if it exists, checks if it is still in use by any user, if yes, fails
-- if not, executes DROP DIRECTORY
DECLARE
v_clusterDirPath varchar2(4000);
v_num_privs NUMBER;
v_grantees VARCHAR2(4000);
v_counter NUMBER := 0;
CURSOR c1 is select distinct(grantee) from sys.all_tab_privs where table_name='${CLUSTER_CONF_DB_DIR}' and privilege IN ('READ','EXECUTE');
BEGIN
select directory_path into v_clusterDirPath from sys.all_directories where directory_name='${CLUSTER_CONF_DB_DIR}';
-- check for permissions
select count(1) into v_num_privs from sys.all_tab_privs where table_name='${CLUSTER_CONF_DB_DIR}' and privilege IN ('READ','EXECUTE');
IF v_num_privs > 0 THEN
  FOR rec in c1
  LOOP
    IF v_counter > 0 THEN
      v_grantees := v_grantees || ',' || rec.grantee;
    ELSE
      v_grantees := rec.grantee;
    END IF;
    v_counter := v_counter + 1;
  END LOOP;
  raise_application_error(-20000, '${CLUSTER_CONF_DB_DIR} is still in use by users - ' || v_grantees);
END IF;
EXCEPTION 
 WHEN NO_DATA_FOUND THEN
   raise_application_error(-20000, 'Directory ${CLUSTER_CONF_DB_DIR} does not exist.');
 WHEN OTHERS THEN
   raise;
END;
/
prompt Dropping directory "${CLUSTER_CONF_DB_DIR}"..
DROP DIRECTORY "${CLUSTER_CONF_DB_DIR}";
prompt The above information is written to ${CLUSTER_CONF_DB_LOG}
spool off;
exit;
EOF
 printstepmsg "Execute the following SQL script while connected as sysdba"
 printstepmsg "to remove cluster config database directory:"
 printstepmsg " @${CLUSTER_CONF_DB_SCRIPT}"
 chmod 755 "${CLUSTER_CONF_DB_SCRIPT}"
}

function remove_cluster_fsdir {
 trap "failstep" INT ERR TERM
 printheader "$STEP: Generating script to remove the filesystem cluster configuration directory"

 if [[ ! -d ${CLUSTER_DIR_PATH} ]]; then
  echo "Specified directory name \"${CLUSTER_DIR_NAME}\" does not exist."  
 else
  cat > "${REMOVE_CLUSTERDIR_SCRIPT}" <<EOF
#!/bin/bash
#generated by ORAKAFKA CLI - remove_cluster.sh
if [[ -d "$CLUSTER_DIR_PATH" ]]; then
 rm -rf "$CLUSTER_DIR_PATH"
 CLUSTER_DIR_DEL_SUC="\$?"
 if [[ "\$CLUSTER_DIR_DEL_SUC" -eq 0 ]]; then
  echo "Removed cluster directory $CLUSTER_DIR_PATH"
 fi
else
 echo "Cluster directory $CLUSTER_DIR_PATH does not exist."
fi
EOF
 chmod 755 "${REMOVE_CLUSTERDIR_SCRIPT}"
 fi
}

source $(dirname $0)/config_util.sh

if [[ $# -eq 0 ]]; then
  usage
fi

debug "Arguments passed to ${SCRIPT_NAME}: $*"

while getopts ":c:sh" opt;
do
  case "$opt" in
    c) CLUSTER_NAME=$OPTARG ;;
    s) DISABLE_SUMMARY=1;;
    h) usage ;;
   \?) echo "Invalid option -$OPTARG"
       usage ;;
    :) echo "Option -$OPTARG requires an argument."
       usage ;;
  esac
done
# after getopts is done, shifts away all the processed options.
shift "$(($OPTIND -1))"

CLUSTER_DIR_NAME=$(validateclustername "${CLUSTER_NAME}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

debug "CLUSTER_DIR_NAME=\"${CLUSTER_DIR_NAME}\""
debug "ORAKAFKA_HOME=${ORAKAFKA_HOME}"

CLUSTER_DIR_PATH="${APP_DATA_CLUSTERS_DIR}/${CLUSTER_DIR_NAME}"

if [[ ! -d "$CLUSTER_DIR_PATH" ]]; then
  failuremsg "Given cluster \"${CLUSTER_DIR_NAME}\" does not exist."
  exit 1
fi

# generate DDL for deletion of the cluster configuration database directory
CLUSTER_CONF_DB_DIR="${CLUSTER_DIR_NAME}"_CONF_DIR
debug "CLUSTER_CONF_DB_DIR=${CLUSTER_CONF_DB_DIR}"
CLUSTER_CONF_DB_SCRIPT="${APP_DATA_SCRATCH_DIR}/orakafka_drop_"${CLUSTER_CONF_DB_DIR}".sql"
CLUSTER_CONF_DB_LOG="${APP_DATA_LOGS_DIR}/orakafka_drop_"${CLUSTER_CONF_DB_DIR}".log"
REMOVE_CLUSTERDIR_SCRIPT="${APP_DATA_SCRATCH_DIR}/remove_cluster_${CLUSTER_DIR_NAME}.sh"

STEP="Step1"
generateddl_remove_dir
printgenscriptstatus "$?" "$STEP"

STEP="Step2"
remove_cluster_fsdir
printgenscriptstatus "$?" "$STEP"

# summary
if [[ -z "$DISABLE_SUMMARY" ]]; then
 echo -e "\n***********SUMMARY************"
 echo -e "\nTODO tasks:\n"
 echo "1. Execute the following SQL while connected as sysdba:"
 echo "   @${CLUSTER_CONF_DB_SCRIPT}"
 echo "2. Execute the following shell script to remove the filesystem cluster directory:"
 echo "   ${REMOVE_CLUSTERDIR_SCRIPT}"
fi
