#!/bin/bash
#
# removeuser_cluster.sh
#
# Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      removeuser_cluster.sh
#
#    DESCRIPTION
#      Revokes permissions on a particular cluster from a DB user.
#      It generates the DDL that revokes privileges on a particular
#      cluster from a specified database user.
#
#      Usage: ./removeuser_cluster.sh -c <cluster-name> -u <db-user> [-h]
#             where options are:
#             -c <cluster-name> - Kafka cluster for which permissions for
#                                 the DB user need to be revoked
#                                 (case-insensitive)
#             -u <db-user>      - DB username (case-insensitive)
#             -h                - Display usage
#
#      Additional options: (not exposed in the usage)
#         -i <userid> : Used to pass the userid generated for the DB username.
#                       This option is used by the setup_all script.
#         -s : This disables displaying the individual step summary.
#              It is used by the setup_all script which runs
#              multiple steps and displays a summary at the end.
#
#    NOTES
#      For debugging, set enviroment variable ORAKAFKA_CONFIG_DEBUG to true.
#      export ORAKAFKA_CONFIG_DEBUG=true
#

SCRIPT_NAME=$(basename $0)

function usage {
 echo "Usage: ${SCRIPT_NAME} -c <cluster-name> -u <db-user> [-h]"
 echo "where options are:"
 echo "-c <cluster-name> - Kafka cluster for which permissions for the DB user"
 echo "                    need to be revoked (case-insensitive)"
 echo "-u <db-user>      - DB username (case-insensitive)"
 echo "-h                - Display usage"
 exit 1
}

function failstep {
  echo "$STEP failed."
  exit 1
}

function removeuser_cluster {
 trap "failstep" INT ERR TERM
 printheader "$STEP: Revoking permissions on cluster configuration directory from \"${DB_USER}\""
 cat > "${REMOVEUSER_CLUSTER_SCRIPT}" <<EOF
--generated by ORAKAFKA CLI - removeuser_cluster.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
spool ${REMOVEUSER_CLUSTER_LOG};
DECLARE
 v_clusterDirPath varchar2(4000);
BEGIN
 select directory_path into v_clusterDirPath from sys.all_directories where directory_name='${CLUSTER_CONF_DB_DIR}';
EXCEPTION
 WHEN NO_DATA_FOUND THEN
   raise_application_error(-20000, 'Directory ${CLUSTER_CONF_DB_DIR} does not exist.');
 WHEN OTHERS THEN 
   RAISE;
 END;
/
prompt Revoking permissions on "${CLUSTER_CONF_DB_DIR}" from "${DB_USER}"
REVOKE READ,EXECUTE ON DIRECTORY "${CLUSTER_CONF_DB_DIR}" from "${DB_USER}";
prompt The above information is written to ${REMOVEUSER_CLUSTER_LOG}
spool off;
exit;
EOF
 printstepmsg "Execute the following script while connected as sysdba"
 printstepmsg "to revoke permissions on cluster conf directory from DB user : "
 printstepmsg " @${REMOVEUSER_CLUSTER_SCRIPT}"
 chmod 755 "${REMOVEUSER_CLUSTER_SCRIPT}"
}

source $(dirname $0)/config_util.sh

# if no arguments are passed, then print usage
if [[ $# -eq 0 ]]; then
  usage
fi

debug "Arguments passed to ${SCRIPT_NAME}: $*"

while getopts ":u:c:i:sh" opt;
do
  case "$opt" in
  c) CLUSTER_NAME=$OPTARG;;
  u) DB_USER=$OPTARG;;
  i) USERID=$OPTARG;;
  s) DISABLE_SUMMARY=1;;
  h) usage;;
 \?) echo "Invalid option -$OPTARG"
     usage ;;
  :) echo "Option -$OPTARG requires an argument."
     usage ;;
  esac
done

CLUSTER_DIR_NAME=$(validateclustername "${CLUSTER_NAME}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

if [[ ! -d "${APP_DATA_CLUSTERS_DIR}/${CLUSTER_DIR_NAME}" ]]; then
 failuremsg "Specified cluster directory ${CLUSTER_DIR_NAME} does not exist."
 exit 1
fi

#validate db user
DB_USER=$(validatedbuser "${DB_USER}")
if [[ "$?" -ne 0 ]]; then
 exit 1
fi

# validate and get userid
USERID=$(validateandgetuserid "${USERID}" "${DB_USER}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

debug "USERID=${USERID}"
debug "CLUSTER_DIR_NAME=${CLUSTER_DIR_NAME}"

# name of cluster DB directory
CLUSTER_CONF_DB_DIR="${CLUSTER_DIR_NAME}"_CONF_DIR

REMOVEUSER_CLUSTER_SCRIPT=${APP_DATA_SCRATCH_DIR}/orakafka_removeuser_cluster_${CLUSTER_DIR_NAME}_${USERID}.sql
REMOVEUSER_CLUSTER_LOG=${APP_DATA_LOGS_DIR}/orakafka_removeuser_cluster_${CLUSTER_DIR_NAME}_${USERID}.log

STEP="Step1"
removeuser_cluster
printgenscriptstatus "$?" "$STEP"

# summary
if [[ -z "$DISABLE_SUMMARY" ]]; then
 echo -e "\n***********SUMMARY************"
 echo -e "\nTODO tasks:\n"
 echo "1. Execute the following SQL while connected as sysdba: "
 echo "   @${REMOVEUSER_CLUSTER_SCRIPT}"
fi

