#!/bin/bash
#
# set_java_home.sh
#
# Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      set_java_home.sh
#
#    DESCRIPTION
#      Configures JAVA_HOME which will be used by other ORA_KAFKA related 
#      scripts that use Java.
#
#      Usage: ./config_java_home.sh -p <java-home-path> [-o]
#             where options are:
#             -p <java-home-path>  - Path of JAVA_HOME
#             -o                   - Override existing JAVA_HOME if already
#                                    configured
#             -h                   - Display usage
# 
#    NOTES
#      For debugging, set environment variable ORAKAFKA_CONFIG_DEBUG to true.
#      export ORAKAFKA_CONFIG_DEBUG=true
#

SCRIPT_NAME=$(basename $0)

function usage {
  echo "Usage: ${SCRIPT_NAME} -p <java-home-path> [-o] [-h]"
  echo "where options are:"
  echo "-p <java-home-path> - Path of JAVA_HOME"
  echo "-o                  - Override existing JAVA_HOME if already configured"
  echo "-h                  - Display usage"
  exit 1
}

function fail {
  printstepmsg "$STEP failed."
  CONFIGURE_JAVA_SCRIPT="$1"
  echo "Cleaning up ${CONFIGURE_JAVA_SCRIPT}.."
  if [[ -f "${CONFIGURE_JAVA_SCRIPT}" ]]; then
    rm -rf ${CONFIGURE_JAVA_SCRIPT}
  fi  
  exit 1
}

function failstep {
  printstepmsg "$STEP failed."
  exit 1
}

function check_java_home {
 trap "failstep" INT ERR TERM
 printheader "$STEP: Check for valid JAVA_HOME"
 JAVA_HOME_PATH="$1"
 if [[ -x "$JAVA_HOME_PATH/bin/java" ]]; then
  printstepmsg "Found $JAVA_HOME_PATH/bin/java, JAVA_HOME path is valid."
  return 0
 else
  printstepmsg "Could not find $JAVA_HOME_PATH/bin/java, JAVA_HOME path is invalid."
  return 1
 fi
}

function check_java_version {
 trap "failstep" INT ERR TERM
 printheader "$STEP: JAVA version check"
 ${JAVA_HOME_PATH}/bin/java -version
 # check if it is a valid JAVA_HOME path by doing a java -version
 JAVA_VERSION=$(${JAVA_HOME_PATH}/bin/java -version 2>&1 | awk -F '"' '/version/ {print $2}')

 if [[ "$JAVA_VERSION" > "1.8" ]]; then
   printstepmsg "Java version >= 1.8"
   return 0
 else
   printstepmsg "FAILURE: Java version < 1.8. Please use Java version >= 1.8"
   return 1
 fi
}

function create_configurejava_script {
 trap "fail ${CONFIGURE_JAVA_SCRIPT}" INT ERR TERM
 printheader "$STEP: Creating configure_java.sh script"
 # write JAVA_HOME to a bash script that can be sourced by
 # other scripts that use Java.
cat > ${CONFIGURE_JAVA_SCRIPT} <<EOF
# generated by ORAKAFKA configure CLI - set_java_home.sh
export JAVA_HOME=${JAVA_HOME_PATH}
EOF
 chmod 755 ${CONFIGURE_JAVA_SCRIPT}
 printstepmsg "Wrote to ${CONFIGURE_JAVA_SCRIPT}"
}

source $(dirname $0)/config_util.sh

if [[ $# -eq 0 ]]; then
  usage
fi

debug "Arguments passed to ${SCRIPT_NAME}: $*"

# set any defaults
OVERRIDE=N

while getopts ":p:oh" opt;
do
  case "$opt" in
   p) JAVA_HOME_PATH=$OPTARG;;
   o) OVERRIDE=Y;;
   h) usage ;;
  \?) echo "Invalid option -$OPTARG"
      usage ;;
   :) echo "Option -$OPTARG requires an argument."
      usage ;;
  esac
done

CONFIGURE_JAVA_SCRIPT=${APP_DATA_SCRIPTS_DIR}/configure_java.sh
 
# check if configure_java.sh script already exists
# if override is not true, then exit otherwise continue
if [[ -f "$CONFIGURE_JAVA_SCRIPT" ]]; then
  echo "JAVA_HOME is already configured in $CONFIGURE_JAVA_SCRIPT"
  if [[ "$OVERRIDE" = "N" ]]; then
    echo "Exiting from ${SCRIPT_NAME}.."
    exit 0
  fi
fi

# check if JAVA_HOME path is not empty
if [[ -z "$JAVA_HOME_PATH" ]]; then
  failuremsg "Invalid or empty JAVA_HOME path."
  exit 1
fi

if [[ ! -d "$JAVA_HOME_PATH" ]]; then
  failuremsg "Specified JAVA_HOME path does not exist."
  exit 1
fi

trap "failstep" INT ERR TERM

STEP="Step1"
check_java_home "$JAVA_HOME_PATH"
printstatus "$?" "$STEP"

STEP="Step2"
check_java_version "$JAVA_HOME_PATH"
printstatus "$?" "$STEP"

STEP="Step3"
create_configurejava_script "${CONFIGURE_JAVA_SCRIPT}"
printstatus "$?" "$STEP"

echo -e "\nSuccessfully configured JAVA_HOME in ${CONFIGURE_JAVA_SCRIPT}"
