#!/bin/bash
#
# setup_all.sh
#
# Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      setup_all.sh
#
#    DESCRIPTION
#      Sets up cluster configuration directories for a default Kafka cluster 
#      and sets up ORA_KAFKA package for a particular DB user.
#      Runs the following steps in order:
#      a) set_java_home
#      b) add_cluster
#      c) install
#      d) adduser_cluster
#
#      Mainly useful for first time setup.
#
#      Usage: ./setup_all.sh -c <cluster-name> -u <db-user> -r <root-location>
#             -p <java-home-path> [-h]
#             where options are:
#             -c <cluster-name>      - Kafka cluster name for which filesystem and database cluster
#                                      configuration directories need to be
#                                      created (case-insensitive)
#             -r <root-location>     - Filesystem location under which location and default dirs are created
#             -u <db-user>           - User for whom ORA_KAFKA package is to be setup (case-insensitive)
#             -p <java-home-path>    - Path for JAVA_HOME            
#             -h                     - Display usage
#
#    NOTES
#      <other useful comments, qualifications, etc.>
#
#

SCRIPT_NAME=$(basename $0)

function usage {
  echo "Usage: $SCRIPT_NAME -c <cluster-name> -u <db-user> -r <root-location> -p <java-home-path> [-h]"
  echo "where options are:"
  echo "-c <cluster-name>      - Kafka cluster name for which filesystem and database cluster"
  echo "                         configuration directories need to be created (case-insensitive)"
  echo "-r <root-location>     - Filesystem location under which location and default dirs are created"
  echo "-u <db-user>           - User for whom ORA_KAFKA package is to be setup (case-insensitive)"
  echo "-p <java-home-path>    - Path for JAVA_HOME"
  echo "-h                     - Display usage"
  exit 1
}

function taskerror {
  echo -e "\n$TASK FAILED."
  exit 1
}

function task3error {
  ORAKAFKA_HOME="$1"
  CLUSTER_DIR_NAME="$2"
  echo -e "\n$TASK FAILED."
  printheader "Rolling back previous tasks.."
  echo -e "\n****Rolling back - TASK 2: Add cluster****"
  ${ORAKAFKA_HOME}/bin/scripts/remove_cluster.sh -c ${CLUSTER_DIR_NAME} 
  exit 1
}

function task4error {
 ORAKAFKA_HOME="$1"
 CLUSTER_DIR_NAME="$2"
 USERID="$3"
 echo -e "\n$TASK FAILED."
 printheader "Rolling back previous tasks.."
 echo -e "\n****Rolling back - TASK 3: Installing ORA_KAFKA package for DB user****"
 ${ORAKAFKA_HOME}/bin/scripts/uninstall.sh -u ${USERID} -s
 echo -e "\n****Rolling back - TASK 2: Add cluster****"
 ${ORAKAFKA_HOME}/bin/scripts/remove_cluster.sh -c ${CLUSTER_DIR_NAME} -s
 # print summary
 cat > "${APP_DATA_SCRATCH_DIR}/rollback_all_${CLUSTER_DIR_NAME}_${USERID}.sql" <<EOF
--generated by ORAKAFKA config CLI - setup_all.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
@${APP_DATA_SCRATCH_DIR}/orakafka_drop_${CLUSTER_DIR_NAME}_CONF_DIR.sql
@${APP_DATA_SCRATCH_DIR}/remove_db_dirs_${USERID}.sql
exit;
EOF
chmod 755 "${APP_DATA_SCRATCH_DIR}/rollback_all_${CLUSTER_DIR_NAME}_${USERID}.sql"
echo -e "\n***********SUMMARY************"
echo -e "\nTODO tasks:\n"

echo "1. Execute the following SQL script in user schema of \"${DB_USER}\""
echo "   @${APP_DATA_SCRATCH_DIR}/uninstall_orakafka_${USERID}.sql"
echo "2. Execute the following SQL script as privileged DBA"
echo "   @${APP_DATA_SCRATCH_DIR}/rollback_all_${CLUSTER_DIR_NAME}_${USERID}.sql"
exit 1
}

if [[ $# -eq 0 ]]; then
 usage
fi

source $(dirname $0)/config_util.sh

while getopts ":c:u:r:p:h" opt;
do
  case "$opt" in
    c) CLUSTER_NAME=$OPTARG;;
    u) DB_USER=$OPTARG;;
    r) ROOT_LOCATION=$OPTARG;;
    p) JAVA_HOME_PATH=$OPTARG;;
    h) usage;;
   \?) echo "Invalid option -$OPTARG"
       usage ;;
    :) echo "Option -$OPTARG requires an argument."
       usage ;;
  esac
done

CLUSTER_DIR_NAME=$(validateclustername "${CLUSTER_NAME}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

if [ -z "$ROOT_LOCATION" ] || [ ! -d "$ROOT_LOCATION" ]; then
 failuremsg "-r <root-location> value for creation of location and default dirs is empty or invalid."
 usage
fi

if [ -z "$JAVA_HOME_PATH" ] || [ ! -d "$JAVA_HOME_PATH" ]; then
 failuremsg "-p <java-home-path> value is empty or invalid."
 usage
fi

DB_USER=$(validatedbuser "${DB_USER}")
if [[ "$?" -ne 0 ]]; then
 failuremsg "-u <db-user> value is empty or invalid."
 exit 1 
fi


# get userid
USERID=$(getuserid "${DB_USER}")
if [[ -z "$USERID" ]]; then
 failuremsg "Empty or invalid user id."
 exit 1
fi

TASK="TASK 1"
trap "taskerror" INT ERR TERM
echo -e "\n*************TASK 1: Set JAVA_HOME*************"
echo -e "\nExecuting script ${ORAKAFKA_HOME}/bin/scripts/set_java_home.sh.."
${ORAKAFKA_HOME}/bin/scripts/set_java_home.sh -p "${JAVA_HOME_PATH}"

TASK="TASK 2"
echo -e "\n*************TASK 2: Add cluster*************"
echo -e "\nExecuting script ${ORAKAFKA_HOME}/bin/scripts/add_cluster.sh.."
${ORAKAFKA_HOME}/bin/scripts/add_cluster.sh -c "${CLUSTER_DIR_NAME}" -s

TASK="TASK 3"
trap "task3error ${ORAKAFKA_HOME} ${CLUSTER_DIR_NAME} ${USERID}" INT ERR TERM
echo -e "\n*************TASK 3: Installing ORA_KAFKA package for DB user*************"
echo -e "\nExecuting script ${ORAKAFKA_HOME}/bin/scripts/install.sh.."
${ORAKAFKA_HOME}/bin/scripts/install.sh -i "${USERID}" -r "${ROOT_LOCATION}" -u "${DB_USER}" -s

TASK="TASK 4"
trap "task4error ${ORAKAFKA_HOME} ${CLUSTER_DIR_NAME} ${USERID}" INT ERR TERM
echo -e "\n************TASK 4: Granting permissions on cluster to DB user************"
echo -e "\nExecuting script ${ORAKAFKA_HOME}/bin/scripts/adduser_cluster.sh.."
${ORAKAFKA_HOME}/bin/scripts/adduser_cluster.sh -i "${USERID}" -c "${CLUSTER_DIR_NAME}" -u "${DB_USER}" -s

SETUP_ALL_SCRIPT="${APP_DATA_SCRATCH_DIR}/setup_all_${CLUSTER_DIR_NAME}_${USERID}.sql"

cat > ${APP_DATA_SCRATCH_DIR}/setup_all_${CLUSTER_DIR_NAME}_${USERID}.sql <<EOF
--generated by ORAKAFKA config CLI - setup_all.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
@${APP_DATA_SCRATCH_DIR}/orakafka_create_${CLUSTER_DIR_NAME}_CONF_DIR.sql
@${APP_DATA_SCRATCH_DIR}/setup_db_dirs_${USERID}.sql
@${APP_DATA_SCRATCH_DIR}/orakafka_adduser_cluster_${CLUSTER_DIR_NAME}_${USERID}.sql
exit;
EOF

chmod 755 "${APP_DATA_SCRATCH_DIR}/setup_all_${CLUSTER_DIR_NAME}_${USERID}.sql"

CLUSTER_DIR_PATH="${APP_DATA_CLUSTERS_DIR}/${CLUSTER_DIR_NAME}"

echo -e "\n***********SUMMARY************"
echo -e "\nTODO tasks:\n"

echo "1. Configure security properties at ${CLUSTER_DIR_PATH}/conf/orakafka.properties"
echo "2. Execute the following SQL while connected as sysdba:"
echo "   @${APP_DATA_SCRATCH_DIR}/setup_all_${CLUSTER_DIR_NAME}_${USERID}.sql"
echo "3. Execute the following SQL in user schema of \"${DB_USER}\":"
echo "   @${APP_DATA_SCRATCH_DIR}/install_orakafka_${USERID}.sql"
