#!/bin/bash
#
# test_cluster.sh
#
# Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      test_cluster.sh - Test for Kafka cluster configuration
#
#    DESCRIPTION
#      Tests the kafka cluster configuration for a specific Kafka
#      cluster specified by the kafka cluster directory name.
#
#      Usage: ./test_cluster.sh [-c <cluster-name> -b <kafka-bootstrap-servers> ]
#                               [-h]
#
#             -c <cluster-name>            - Name of cluster directory
#                                            created under Oracle SQL Access to
#                                            Kafka kit
#                                            (case-insensitive)
#             -b <kafka-bootstrap-servers> - Comma separated list of Kafka
#                                            cluster bootstrap servers
#             -h                           - Display usage
#
#      This test will run "list topics" against the Kafka cluster
#      specified by "bootstrap servers" with the applicable security properties.
#
#      The output files are generated under the directory <cluster-name>/logs.
#      Each time this script is run, a log file named 
#      test_<cluster-name>.log.<TIME> is generated.
#      The log file has the output of the test that runs "list topics" against
#      the given Kafka cluster using ORA_KAFKA.
# 
#      User model: This script must be run by the "oracle" user.
#
#    NOTES
#      For debugging, set enviroment variable ORAKAFKA_CONFIG_DEBUG to true.
#      export ORAKAFKA_CONFIG_DEBUG=true

SCRIPT_NAME=$(basename $0)

# Prints usage to the terminal and exits
function usage() {
  echo "Usage: ${SCRIPT_NAME} [-c <cluster-name> -b <kafka-bootstrap-servers>] [-h]"
  echo "where options are:"
  echo "-c <cluster-name>            - Name of the cluster directory created under"
  echo "                               Oracle SQL Access to Kafka kit (case-insensitive)"
  echo "-b <kafka-bootstrap-servers> - Comma separated list of kafka cluster bootstrap servers"
  echo "-h                           - Display usage"
  exit 1
}

function debug {
  if [[ "${ORAKAFKA_CONFIG_DEBUG}" == true ]]; then
    # if a debug message is set, echo the message
    if [[ ! -z "$1" ]]; then
      echo "$1"
    fi
  fi
}

function cleanup_loc_file {
  LOC_FILE_PATH=$1
  if [[ -f "${LOC_FILE_PATH}" ]]; then
    rm -rf "${LOC_FILE_PATH}";
  fi
}

source $(dirname $0)/config_util.sh

# if no arguments are passed, then print usage
if [[ $# -eq 0 ]]; then
  usage
fi

debug "Arguments passed to ${SCRIPT_NAME}: $*"

while getopts ":c:b:h" opt;
do
  case "$opt" in
    c) CLUSTER_NAME=$OPTARG ;;    
    b) KAFKA_BOOTSTRAP_SERVERS=$OPTARG ;;
    h) usage ;;
   \?) echo "Invalid option -$OPTARG"
       usage ;;
    :) echo "Option -$OPTARG requires an argument."
       usage ;;
  esac
done

CLUSTER_DIR_NAME=$(validateclustername "${CLUSTER_NAME}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

# trims any leading or trailing spaces in the given bootstrap servers
KAFKA_BOOTSTRAP_SERVERS=$(printf "$KAFKA_BOOTSTRAP_SERVERS" |  sed -e 's/^[[:space:]]*//;s/[[:space:]]*$//')

if [[ -z "$KAFKA_BOOTSTRAP_SERVERS" ]]; then
  echo "Kafka bootstrap servers is either empty or invalid."
  usage
fi

debug "CLUSTER_DIR_NAME=\"${CLUSTER_DIR_NAME}\""
debug "KAFKA_BOOTSTRAP_SERVERS=\"${KAFKA_BOOTSTRAP_SERVERS}\""
debug "APP_DATA_CLUSTERS_DIR=\"${APP_DATA_CLUSTERS_DIR}\""

CLUSTER_DIR="${APP_DATA_CLUSTERS_DIR}/${CLUSTER_DIR_NAME}"
CLUSTER_LOG_DIR="${CLUSTER_DIR}/logs"

# check if the cluster directory exists
if [[ ! -d "${CLUSTER_DIR}" ]]; then
  echo "The given cluster directory \"${CLUSTER_DIR_NAME}\" does not exist under ${APP_DATA_CLUSTERS_DIR}."
  echo "Please re-run cluster config test with a configured cluster directory under ${APP_DATA_CLUSTERS_DIR}"
  exit 1
fi

debug "CLUSTER_DIR=\"${CLUSTER_DIR}\""
debug "CLUSTER_LOG_DIR=\"${CLUSTER_LOG_DIR}\""

# create a temporary location file with operation "list topics"
# and bootstrap servers given as input to this script.
TMP_LOC_FILE="${CLUSTER_LOG_DIR}"/orakafka_stream_"${CLUSTER_DIR_NAME}".loc
echo "ora_kafka_operation=metadata" > "${TMP_LOC_FILE}"
echo "kafka_bootstrap_servers=${KAFKA_BOOTSTRAP_SERVERS}" >> "${TMP_LOC_FILE}"

# remove the temporary location file upon exit.
trap "cleanup_loc_file \"${TMP_LOC_FILE}\"" EXIT

# Run the cluster config test with the temporary location file
CURRENT_TIME=$(date "+%Y.%m.%d-%H.%M.%S")
CLUSTER_CONFIG_TEST_LOG="$CLUSTER_LOG_DIR"/test_"${CLUSTER_DIR_NAME}".log."$CURRENT_TIME"

# Write information about test run to the cluster config test log file.
echo "Kafka cluster configuration test - \"${CLUSTER_DIR_NAME}\"
------------------------------------------------------
KAFKA_BOOTSTRAP_SERVERS=${KAFKA_BOOTSTRAP_SERVERS}
LOCATION_FILE=\"${TMP_LOC_FILE}\"" > "${CLUSTER_CONFIG_TEST_LOG}"
# write location file content
while read -r line;
do
  echo "$line">> "${CLUSTER_CONFIG_TEST_LOG}"
done < "${TMP_LOC_FILE}"
echo -e "\nList topics output:" >> "${CLUSTER_CONFIG_TEST_LOG}"
# run orakafka_stream.sh with temp location file.
"${CLUSTER_DIR}/orakafka_stream.sh" "${TMP_LOC_FILE}" >> "${CLUSTER_CONFIG_TEST_LOG}"
let status=$?

echo -e "\n" >> "${CLUSTER_CONFIG_TEST_LOG}"
if [ "$status" -ne 0 ] ; then
  echo -e "Kafka cluster configuration test failed.\n" | tee -a "${CLUSTER_CONFIG_TEST_LOG}"
else
  echo -e "Kafka cluster configuration test succeeded.\n" | tee -a "${CLUSTER_CONFIG_TEST_LOG}"
fi
echo "Please look at \"${TMP_LOC_FILE}.pp.out\" for detailed log." >> "${CLUSTER_CONFIG_TEST_LOG}"

debug "Wrote output of Kafka cluster configuration test to ${CLUSTER_CONFIG_TEST_LOG}"

# set permissions to 600 for files under cluster logs directory
chmod 600 "${CLUSTER_LOG_DIR}"/*

# terminal output
head -n 10 ${CLUSTER_CONFIG_TEST_LOG}
echo "..."
echo "For full log, please look at ${CLUSTER_CONFIG_TEST_LOG}"

echo "Test of Kafka cluster configuration for \"${CLUSTER_DIR_NAME}\" completed."
