#!/bin/bash
#
# test_views.sh
#
# Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      test_views.sh
#
#    DESCRIPTION
#      Tests Oracle SQL Access to Kafka end-to-end using the cluster 
#      that has been configured and the DB user in whose schema 
#      ORA_KAFKA package has been setup.
#      Generates the DDL to perform each of the following steps:     
#      a) In the user schema that has ORA_KAFKA package setup, 
#         registers the cluster that has already been configured
#         by add_cluster and create views for given topic.
#      b) Runs a SQL query to do a "select count(*) from <view>"
#         in the user schema.
#      c) Cleans up the test setup from user schema
#
#      Usage: ./test_views.sh -u <db-user> -c <cluster-name> -t <topic-name> -b <bootstrap-servers>
#      where options are:
#      -u <db-user>           - DB user in whose schema ORA_KAFKA package is already 
#                               setup (case-insensitive)
#      -c <cluster-name>      - Configured cluster name (case-sensitive)
#      -t <topic-name>        - Name of the topic to be used in test
#      -b <bootstrap-servers> - Comma separated list of kafka bootstrap servers
#      -h                     - Display usage
#
#    NOTES
#      For debugging, set environment variable ORAKAFKA_CONFIG_DEBUG to true.
#      export ORAKAFKA_CONFIG_DEBUG=true
#

SCRIPT_NAME=$(basename $0)

function usage {
  echo "Usage: ${SCRIPT_NAME} -u <db-user> -c <cluster-name> -t <topic-name> -b <bootstrap-servers> [-h]"
  echo "where options are:"
  echo "-u <db-user>           - DB user in whose schema ORA_KAFKA package is"
  echo "                         already setup (case-insensitive)"
  echo "-c <cluster-name>      - Configured cluster name (case-sensitive)"
  echo "-t <topic-name>        - Name of the topic to be used in test"
  echo "-b <bootstrap-servers> - Comma separated list of kafka bootstrap servers"
  echo "-h                     - Display usage"
  exit 1
}

function cleanup {
 trap "failstep" INT ERR TERM 
 printheader "$STEP: Generate DDL to cleanup test setup from user schema \"${DB_USER}\""
 # delete reference table and drop cluster if exist in user schema

 cat > "${TEST_ORAKAFKA_CLEANUP_SCRIPT}" <<EOF
--generated by ORAKAFKA config CLI - test_views.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
spool ${TEST_ORAKAFKA_CLEANUP_SCRIPT_LOG};
prompt Verifying user schema..
DECLARE
 v_valid_schema NUMBER;
BEGIN
 select 1 into v_valid_schema from sys.dual where SYS_CONTEXT('USERENV', 'CURRENT_SCHEMA')='${DB_USER}';
EXCEPTION
 WHEN NO_DATA_FOUND THEN
   raise_application_error(-20000, 'Current schema is not "${DB_USER}".');
END;
/
BEGIN
execute immediate 'DROP TABLE ORAKAFKA_VARCHAR2_REF_TABLE PURGE';
sys.dbms_output.put_line('Dropped reference table ORAKAFKA_VARCHAR2_REF_TABLE');
END;
/
BEGIN
ORA_KAFKA.DROP_CLUSTER('${CLUSTER_DIR_NAME}', true);
sys.dbms_output.put_line('Dropped cluster ${CLUSTER_DIR_NAME}');
END;
/
prompt Cleanup of test_ora_kafka done.
prompt The above information is written to ${TEST_ORAKAFKA_CLEANUP_SCRIPT_LOG}
spool off;
exit;
EOF
 printstepmsg "Execute the following script in user schema to cleanup test"
 printstepmsg "setup for Oracle SQL Access to Kafka:"
 printstepmsg " @${TEST_ORAKAFKA_CLEANUP_SCRIPT}"
 chmod 755 "${TEST_ORAKAFKA_CLEANUP_SCRIPT}"   
}

function failstep {
 echo "$STEP failed."
 exit 1
}

function validate_user {
 trap "failstep" INT ERR TERM
 printheader "$STEP: Generate DDL to validate the given DB user \"${DB_USER}\""
 #check if user is valid and has ORA_KAFKA installed
 cat > "${VALIDATE_ORAKAFKA_USER_SCRIPT}" <<EOF
--generated by ORAKAFKA config CLI - test_views.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
spool ${VALIDATE_ORAKAFKA_USER_LOG};
DECLARE
 v_userValue varchar2(200) ;
 v_numObjects NUMBER;
BEGIN
 sys.dbms_output.put_line('Checking if user ${DB_USER} exists..');
 BEGIN
  select sys.dbms_assert.schema_name('${DB_USER}') into  v_userValue from sys.dual;
 EXCEPTION
  WHEN sys.dbms_assert.INVALID_SCHEMA_NAME THEN 
    raise_application_error(-20000, 'User ${DB_USER} does not exist.');
 END;
 sys.dbms_output.put_line('Checking if ORA_KAFKA package is found in user schema..');
 BEGIN
  -- package and package body
  select count(1) into v_numObjects from sys.all_objects where object_name='ORA_KAFKA' and owner='${DB_USER}';
  IF v_numObjects < 2 THEN
    raise_application_error(-20000, 'Expected to find ORA_KAFKA package and package body in user schema');
  END IF;
 EXCEPTION
  WHEN NO_DATA_FOUND THEN
    raise_application_error(-20000, 'ORA_KAFKA package not found in user schema.');
 END;
END;
/
prompt Validation of user ${DB_USER} done.
prompt The above information is written to ${VALIDATE_ORAKAFKA_USER_LOG}
spool off;
exit;
EOF
 printstepmsg "Execute the following SQL script connected as sysdba to validate user:"
 printstepmsg " @${VALIDATE_ORAKAFKA_USER_SCRIPT}"
 chmod 755 "${VALIDATE_ORAKAFKA_USER_SCRIPT}"
}

function test_orakafka {
 trap "failstep" INT ERR TERM
 printheader "$STEP: Generate DDL to test Oracle SQL Access to Kafka end-to-end for user schema \"${DB_USER}\""
 CREATE_REF_TABLE="CREATE TABLE ORAKAFKA_VARCHAR2_REF_TABLE
 (c1 VARCHAR2(4000));"
REGISTER_CLUSTER="BEGIN
ORA_KAFKA.REGISTER_CLUSTER('${CLUSTER_DIR_NAME}','${BOOTSTRAP_SERVERS}','${DB_USER}_KAFKA_DEF_DIR',
'${DB_USER}_KAFKA_LOC_DIR', '${CLUSTER_DIR_NAME}_CONF_DIR', 'Test orakafka cluster');
sys.dbms_output.put_line('Registered cluster ${CLUSTER_DIR_NAME}');
END;
/"
CREATE_VIEWS="DECLARE
v_views_created INTEGER;
v_application_id VARCHAR2(128);
BEGIN
ORA_KAFKA.CREATE_VIEWS('${CLUSTER_DIR_NAME}','orakafkatestgroup','${TOPIC_NAME}','CSV','ORAKAFKA_VARCHAR2_REF_TABLE',
v_views_created,v_application_id, 1, TRUE, '');
sys.dbms_output.put_line('View created = ' || v_views_created);
sys.dbms_output.put_line('Application id = ' || v_application_id);
sys.dbms_output.put_line('Created views for ${TOPIC_NAME}');
END;
/
"
cat > "${TEST_ORAKAFKA_VIEWS_SCRIPT}" <<EOF
--generated by ORAKAFKA config CLI - test_views.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
spool ${TEST_ORAKAFKA_VIEWS_SCRIPT_LOG};
prompt Verifying user schema..
DECLARE
 v_valid_schema NUMBER;
BEGIN
 select 1 into v_valid_schema from sys.dual where SYS_CONTEXT('USERENV', 'CURRENT_SCHEMA')='${DB_USER}';
EXCEPTION
 WHEN NO_DATA_FOUND THEN
   raise_application_error(-20000, 'Current schema is not "${DB_USER}".');
END;
/
${CREATE_REF_TABLE}
${REGISTER_CLUSTER}
${CREATE_VIEWS}
DECLARE
 v_count NUMBER;
BEGIN
 select count(*) into v_count from KV_${CLUSTER_DIR_NAME}_orakafkatestgroup_${SQL_TOPIC_NAME}_0; 
 sys.dbms_output.put_line('Row count = ' || v_count);
 sys.dbms_output.put_line('Please verify the row count.');
EXCEPTION
 WHEN NO_DATA_FOUND THEN
  raise_application_error(-20000, 'Error fetching from view KV_${CLUSTER_DIR_NAME}_orakafkatestgroup_${SQL_TOPIC_NAME}_0');
END;
/
spool off;
prompt The above information is written to ${TEST_ORAKAFKA_VIEWS_SCRIPT_LOG}
exit;
EOF
 printstepmsg "Execute the following script in user schema to test "
 printstepmsg "Oracle SQL Access to Kafka:"
 printstepmsg " @${TEST_ORAKAFKA_VIEWS_SCRIPT}"
 chmod 755 "${TEST_ORAKAFKA_VIEWS_SCRIPT}"
}

source $(dirname $0)/config_util.sh

if [[ $# -eq 0 ]]; then
 usage
fi

debug "Arguments passed to ${SCRIPT_NAME}: $*"

while getopts ":u:c:t:b:h" opt;
do
 case "$opt" in
   u) DB_USER=$OPTARG;;
   c) CLUSTER_NAME=$OPTARG;;
   t) TOPIC_NAME=$OPTARG;;
   b) BOOTSTRAP_SERVERS=$OPTARG;;
   h) usage;;
  \?) echo "Invalid option -$OPTARG"
      usage ;;
   :) echo "Option -$OPTARG requires an argument."
      usage ;;
 esac
done

DB_USER=$(validatedbuser "${DB_USER}")
if [[ "$?" -ne 0 ]]; then
 exit 1
fi

# get userid
USERID=$(getuserid "${DB_USER}")
if [[ -z "$USERID" ]]; then
 failuremsg "Empty or invalid user id."
 exit 1
fi

CLUSTER_DIR_NAME=$(validateclustername "${CLUSTER_NAME}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

debug "USERID=${USERID}"
debug "CLUSTER_DIR_NAME=${CLUSTER_DIR_NAME}"

# check if cluster fs directory exists
if [[ ! -d "${APP_DATA_CLUSTERS_DIR}/${CLUSTER_DIR_NAME}" ]]; then
 failuremsg "Specified cluster directory ${CLUSTER_DIR_NAME} does not exist."
 exit 1
fi

if [ -z "$TOPIC_NAME" ] || [ -z "$BOOTSTRAP_SERVERS" ]; then
 echo "-t <topic-name> or -b <bootstrap-servers> is empty or invalid."
 usage
fi

#NEDCBD-179: TOPIC_NAME shows up in ORACLE view names with some changes:
# a-z -> A-Z
# .   -> #
# -   -> #
SQL_TOPIC_NAME=`echo ${TOPIC_NAME} | tr '[a-z.\-]' '[A-Z##]'`


# main steps in testing Oracle SQL Access to Kafka end-to-end
# check if user is valid and has ORA_KAFKA package installed
# run end to end test and cleanup

VALIDATE_ORAKAFKA_USER_SCRIPT=${APP_DATA_SCRATCH_DIR}/test_orakafka_validate_${USERID}.sql
VALIDATE_ORAKAFKA_USER_LOG=${APP_DATA_LOGS_DIR}/test_orakafka_validate_${USERID}.log

STEP="Step1"
validate_user
printgenscriptstatus "$?" "$STEP"
        
TEST_ORAKAFKA_VIEWS_SCRIPT=${APP_DATA_SCRATCH_DIR}/test_orakafka_views_${USERID}_${TOPIC_NAME}.sql
TEST_ORAKAFKA_VIEWS_SCRIPT_LOG=${APP_DATA_LOGS_DIR}/test_orakafka_views_${USERID}_${TOPIC_NAME}.log

STEP="Step2"
test_orakafka
printgenscriptstatus "$?" "$STEP"

TEST_ORAKAFKA_CLEANUP_SCRIPT=${APP_DATA_SCRATCH_DIR}/test_orakafka_views_cleanup_${USERID}_${TOPIC_NAME}.sql
TEST_ORAKAFKA_CLEANUP_SCRIPT_LOG=${APP_DATA_LOGS_DIR}/test_orakafka_views_cleanup_${USERID}_${TOPIC_NAME}.log

STEP="Step3"
cleanup
printgenscriptstatus "$?" "$STEP"

echo -e "\n***********SUMMARY************"
echo -e "\nTODO tasks:\n"

echo "1. Execute the following SQL while connected as sysdba:"
echo "   @${VALIDATE_ORAKAFKA_USER_SCRIPT}"
echo "2. Execute the following SQL in user schema of \"${DB_USER}\" to test"
echo "   Oracle SQL Access to Kafka:"
echo "   @${TEST_ORAKAFKA_VIEWS_SCRIPT}"
echo "3. Execute the following SQL in user schema of \"${DB_USER}\" to cleanup test setup:"
echo "   @${TEST_ORAKAFKA_CLEANUP_SCRIPT}"

