#!/bin/bash
#
# uninstall.sh
#
# Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      uninstall.sh
#
#    DESCRIPTION
#      Uninstalls ORA_KAFKA package by running the following steps:
#      Generates DDL for:
#      a) Dropping all the clusters registered in user schema
#      b) Dropping the location and default dirs for the user
#      c) Uninstalling ORA_KAFKA package from user schema
#      and
#      d) Deletes the filesystem location and default directories
#         created for the user
#
#      Usage: ./uninstall.sh -u <db-user> [-h]
#             where options are:
#             -u <db-user>       - User schema from which ORA_KAFKA package has to
#                                  be uninstalled (case-insensitive)
#             -h                 - Display usage
#
#      Additional options: (not exposed in the usage)
#         -i <userid> : Used to pass the userid generated for the DB username.
#                       This option is used by the setup_all script.
#         -s : This disables displaying the individual step summary.
#              It is used by the setup_all script which runs
#              multiple steps and displays a summary at the end.
#
#    NOTES
#      For debugging, set environment variable ORAKAFKA_CONFIG_DEBUG to true.
#      export ORAKAFKA_CONFIG_DEBUG=true
#

SCRIPT_NAME=$(basename $0)

function usage {
  echo "Usage: ${SCRIPT_NAME} -u <db-user> [-h]"
  echo "where options are:"
  echo "-u <db-user>  - User schema from which ORA_KAFKA package has to"
  echo "                be uninstalled (case-insensitive)"
  echo "-h            - Display usage"
  exit 1
}

function failstep {
  echo "$STEP failed."
  exit 1
}

function uninstall_orakafka_user {
 trap "failstep" INT ERR TERM
 printheader "$STEP: Uninstall ORA_KAFKA package from user schema"
 cat > "${UNINSTALL_ORA_KAFKA_SCRIPT}" <<EOF
--generated by ORAKAFKA config CLI - uninstall.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
spool ${UNINSTALL_ORA_KAFKA_LOG};
-- verifying user
prompt Verifying user schema..
DECLARE
 v_valid_schema NUMBER;
BEGIN
 select 1 into v_valid_schema from sys.dual where SYS_CONTEXT('USERENV', 'SESSION_USER')='${DB_USER}';
EXCEPTION
 WHEN NO_DATA_FOUND THEN
   raise_application_error(-20000, 'User schema is not "${DB_USER}".');
END;
/
-- package and package body
prompt Verifying if ORA_KAFKA package is installed..
DECLARE
v_numObjects NUMBER;
BEGIN
select count(1) into v_numObjects from sys.all_objects where object_name='ORA_KAFKA' and owner='${DB_USER}';
IF v_numObjects = 0 THEN
 raise_application_error(-20000, 'ORA_KAFKA package not found in user schema');
END IF;
END;
/
prompt Dropping any registered clusters in user schema..
DECLARE
 CURSOR c1 is select cluster_name from ora_kafka_cluster;
BEGIN
 FOR rec in c1
 LOOP
  ORA_KAFKA.DROP_CLUSTER(rec.cluster_name, true);
 END LOOP;
END;
/
-- uninstall ORA_KAFKA package from user schema
prompt Uninstalling ORA_KAFKA package from user schema..
@${ORAKAFKA_SQL_DIR}/catnoorakafka.sql
prompt Uninstall of ORA_KAFKA package from user schema completed.
prompt The above information is written to ${UNINSTALL_ORA_KAFKA_LOG}
spool off;
exit;
EOF

 printstepmsg "Execute the following SQL script to uninstall ORA_KAFKA package from user schema:"
 printstepmsg " ${UNINSTALL_ORA_KAFKA_SCRIPT}"
 chmod 755 ${UNINSTALL_ORA_KAFKA_SCRIPT}
}

function generateddl_drop_locdef_db_dirs { 
  trap "failstep" INT ERR TERM
  printheader "$STEP: Drop database directories for the user and revoke permissions on DB dirs"
CHECK_USER_EXISTS="-- check if the user exists
DECLARE
v_userValue varchar2(200) ;
BEGIN
 select sys.dbms_assert.schema_name('${DB_USER}') into v_userValue from sys.dual;
EXCEPTION 
WHEN sys.dbms_assert.INVALID_SCHEMA_NAME THEN
  raise_application_error(-20001, 'User ${DB_USER} does not exist');
END;
/"
cat > "${REMOVE_DB_DIRS_SCRIPT}" <<EOF
--generated by ORAKAFKA config CLI - uninstall.sh
set verify off;
set feedback 1;
set termout on;
set serveroutput on;
WHENEVER OSERROR EXIT 9;
WHENEVER SQLERROR EXIT SQL.SQLCODE;
spool ${REMOVE_DB_DIRS_LOG};
prompt Checking if the user exists..
${CHECK_USER_EXISTS}
prompt Revoking permissions on "${LOC_DB_DIR}" and "${DEF_DB_DIR}"
REVOKE READ, WRITE ON DIRECTORY "${LOC_DB_DIR}" from "${DB_USER}";
REVOKE READ, WRITE ON DIRECTORY "${DEF_DB_DIR}" from "${DB_USER}";
prompt Dropping location directory "${LOC_DB_DIR}"..
DROP DIRECTORY "${LOC_DB_DIR}";
prompt Dropping default directory "${DEF_DB_DIR}"..
DROP DIRECTORY "${DEF_DB_DIR}";
prompt The above information is written to ${REMOVE_DB_DIRS_LOG}
spool off;
exit;
EOF

  printstepmsg "Execute the following script while connected as sysdba"
  printstepmsg "to drop location and default directories:"
  printstepmsg " @${REMOVE_DB_DIRS_SCRIPT}"
  chmod 755 ${REMOVE_DB_DIRS_SCRIPT}
}

function remove_locdef_fsdirs {
 trap "failstep" INT ERR TERM
 printheader "$STEP: Remove location and default filesystem directories"
 ${APP_DATA_SCRATCH_DIR}/remove_fsdirs_${USERID}.sh 
}


source $(dirname $0)/config_util.sh

# if no arguments are passed, print usage
if [[ $# -eq 0 ]]; then
 usage
fi

debug "Arguments passed to ${SCRIPT_NAME}: $*"

while getopts ":u:i:sh" opt;
do
 case "$opt" in
  u) DB_USER=$OPTARG;;
  i) USERID=$OPTARG;;
  s) DISABLE_SUMMARY=1;;
  h) usage;;
 \?) echo "Invalid option -$OPTARG"
     usage ;;
  :) echo "Option -$OPTARG requires an argument."
     usage ;;
 esac
done
# after getopts is done, shifts away all the processed options.
shift "$(($OPTIND -1))"

#validate db user
DB_USER=$(validatedbuser "${DB_USER}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

# validate and get userid
USERID=$(validateandgetuserid "${USERID}" "${DB_USER}")
if [[ "$?" -ne 0 ]]; then
  exit 1
fi

debug "USERID=${USERID}"

# generate DDL for dropping clusters and uninstall ORA_KAFKA package from user schema
UNINSTALL_ORA_KAFKA_SCRIPT=${APP_DATA_SCRATCH_DIR}/uninstall_orakafka_${USERID}.sql
UNINSTALL_ORA_KAFKA_LOG=${APP_DATA_LOGS_DIR}/uninstall_orakafka_${USERID}.log

STEP="Step1"
uninstall_orakafka_user
printgenscriptstatus "$?" "$STEP"

REMOVE_DB_DIRS_SCRIPT=${APP_DATA_SCRATCH_DIR}/remove_db_dirs_${USERID}.sql
REMOVE_DB_DIRS_LOG=${APP_DATA_LOGS_DIR}/remove_db_dirs_${USERID}.log
LOC_DB_DIR="${DB_USER}"_KAFKA_LOC_DIR
DEF_DB_DIR="${DB_USER}"_KAFKA_DEF_DIR

STEP="Step2"
generateddl_drop_locdef_db_dirs
printgenscriptstatus "$?" "$STEP"

STEP="Step3"
remove_locdef_fsdirs
printstatus "$?" "$STEP"

if [[ -z "$DISABLE_SUMMARY" ]]; then
 echo -e "\n***********SUMMARY************"
 echo -e "\nTODO tasks:\n"
 echo "1. Execute the following SQL in user schema: "
 echo "   @${UNINSTALL_ORA_KAFKA_SCRIPT}"
 echo "2. Execute the following SQL while connected as sysdba: "
 echo "   @${REMOVE_DB_DIRS_SCRIPT}"
fi
