#!/bin/bash
#
# verify_install.sh
#
# Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      verify_install.sh
#
#    DESCRIPTION
#
#      Verify install script for orakafka 
#      Performs the following checks:
#      a) All files/directories are owned by oracle:oinstall (default)
#         or the specified owner/group 
#      b) Checks that files and directories are created with proper
#         permissions
#
#       Usage: ./verify_install.sh [-a <app-data-home>] [-u <owner>:<group>] [-h]
#       where options are:
#       -a <app-data-home> - absolute path of app_data/ directory 
#                            default: <ora-kafka-home>/app_data
#                            <ora-kafka-home> directory is auto-detected  
#                            by this script
#
#       -u <owner>:<group> - colon separated file owner and group
#                            The expected file owner and group 
#                            for verification purposes
#                            default value: oracle:oinstall
#       -h          - Display usage
#
set -e

SCRIPT_NAME=$(basename $0)

function usage {
  echo "Usage: ${SCRIPT_NAME} [-a <app-data-home>] [-u <owner>:<group>] [-h]"
  echo "where options are:"
  echo "-a <app-data-home> - absolute path of app_data/ directory" 
  echo "                     default: <ora-kafka-home>/app_data"
  echo "                     <ora-kafka-home> directory is auto-detected"  
  echo "                     by this script"
  echo "                                   "       
  echo "-u <owner>:<group> - colon separated file owner and group"
  echo "                     The expected file owner and group" 
  echo "                     for verification purposes"
  echo "                     default: oracle:oinstall"
  echo "-h                 - Display usage"
  exit 1
}


source $(dirname $0)/config_util.sh

debug "Arguments passed to ${SCRIPT_NAME}: $*"

while getopts ":a:u:h" opt;
do
  case "$opt" in
   a) APP_DATA_DIR=$OPTARG;;
   u) FILE_OWNER_GRP=$OPTARG;;
   h) usage ;;
  \?) echo "Invalid option -$OPTARG"
      usage ;;
   :) echo "Option -$OPTARG requires an argument."
      usage ;;
  esac
done

# after getopts is done, shifts away all the processed options.
shift "$(($OPTIND -1))"


 #enclose  the owner:group with colons to prevent partial matches in grep
if [[ -z "${FILE_OWNER_GRP}" ]]; then
  FILE_OWNER_GRP="oracle:oinstall"
fi
#delimit with colon so there are no partial matches in grep
DLTD_FILE_OWNER_GRP=":${FILE_OWNER_GRP}:"

if [[ -z "${APP_DATA_DIR}" ]]; then
  #get the default value set by config_util.sh 
  APP_DATA_DIR="${APP_DATA_HOME}"
fi


KIT_HOME="${ORAKAFKA_HOME}"
ret_code=0

#verify that all files/dirs are owned by oracle:oinstall or specified user/group
#we expect no output from the following command in the 'good' case
IFS=$'\n'
declare -a OINSTALL_DIRS=($( cd "${PRODUCT_HOME}"; find  -printf ':%u:%g:%p\n' | grep -ve "${DLTD_FILE_OWNER_GRP}"));
if [[ ${#OINSTALL_DIRS[@]} -ne 0 ]]; 
then
  echo "Check all files/dirs ownership - failed"
  echo ".....Found dirs/files not owned by \"${FILE_OWNER_GRP}\""
  ret_code=1
else 
  echo "Check all files/dirs ownership - passed"
fi

#
# exclude the app_data/ if under PRODUCT_HOME as it contains changeable user data
# note: app_data/ may be outside PRODUCT_HOME; which is excluded for dir priv check
#
# Verify only the dir privs  
#
declare -a BAD_PRIV_DIRS=($(find "${PRODUCT_HOME}" -type d -not -path "${APP_DATA_DIR}/*" \
       -exec ls -lad {} \; \
       | grep  -ve 'drwxr-xr-x' \
       | awk '{print $1" "$9;}')) 
if [[ ${#BAD_PRIV_DIRS[@]} -ne 0 ]]; 
then
  echo "Check directory privileges - failed"
  echo ".....Found dirs with wrong permissions: "
  echo ".....${BAD_PRIV_DIRS[*]}"
  ret_code=1
 else
  echo "Check directory privileges - passed"
fi

#find files with wrong permissions
#This find command will output the following files as executables
#Expected executables are included in EXPECTED_EXE_FILES array.
declare -a ACTUAL_EXE_FILES=($(cd "${KIT_HOME}" ; \
   find -type f  \
   -exec ls -la  {} \; \
   | grep -e '-rwxr-xr-x' \
   | awk '{print $1" "$9;}' | sort))

declare -a EXPECTED_EXE_FILES=(  
          '-rwxr-xr-x ./bin/orakafka.sh'
          '-rwxr-xr-x ./bin/orakafka_stream.sh'
          '-rwxr-xr-x ./bin/scripts/add_cluster.sh'
          '-rwxr-xr-x ./bin/scripts/adduser_cluster.sh'
          '-rwxr-xr-x ./bin/scripts/config_util.sh'
          '-rwxr-xr-x ./bin/scripts/install.sh'
          '-rwxr-xr-x ./bin/scripts/list_clusters.sh'
          '-rwxr-xr-x ./bin/scripts/remove_cluster.sh'
          '-rwxr-xr-x ./bin/scripts/removeuser_cluster.sh'
          '-rwxr-xr-x ./bin/scripts/set_java_home.sh'
          '-rwxr-xr-x ./bin/scripts/setup_all.sh'
          '-rwxr-xr-x ./bin/scripts/test_cluster.sh'
          '-rwxr-xr-x ./bin/scripts/test_views.sh'
          '-rwxr-xr-x ./bin/scripts/uninstall.sh'
          '-rwxr-xr-x ./bin/scripts/verify_install.sh');

if [[ ${#ACTUAL_EXE_FILES[@]} -ne ${#EXPECTED_EXE_FILES[@]} ]]; 
then
  echo "Check expected executable files - failed"
  echo ".....Expected:${#EXPECTED_EXE_FILES[@]} Actual:${#ACTUAL_EXE_FILES[@]}"
  echo "Expected file listing: "
  echo "${EXPECTED_EXE_FILES[*]}"
  echo "Actual file listing: "
  echo "${ACTUAL_EXE_FILES[*]}"
  ret_code=1
else
  echo "Check expected executable files - passed"
fi
unset IFS

exit $ret_code
